@@ -29,12 +29,12 @@
  * <refsect2>
  * <title>Example pipelines</title>
  * |[
- * gst-launch -v audiotestsrc wave=sine num-buffers=100 ! audioconvert ! vorbisenc ! oggmux ! filesink location=sine.ogg
+ * gst-launch-1.0 -v audiotestsrc wave=sine num-buffers=100 ! audioconvert ! vorbisenc ! oggmux ! filesink location=sine.ogg
  * ]| Encode a test sine signal to Ogg/Vorbis.  Note that the resulting file
  * will be really small because a sine signal compresses very well.
  * |[
- * gst-launch -v alsasrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=alsasrc.ogg
- * ]| Record from a sound card using ALSA and encode to Ogg/Vorbis.
+ * gst-launch-1.0 -v autoaudiosrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=alsasrc.ogg
+ * ]| Record from a sound card and encode to Ogg/Vorbis.
  * </refsect2>
  */
 #ifdef HAVE_CONFIG_H
@@ -616,16 +616,17 @@ gst_vorbis_enc_sink_event (GstAudioEncoder * enc, GstEvent * event)
 /*
  * (really really) FIXME: move into core (dixit tpm)
  */
-/**
+/*
  * _gst_caps_set_buffer_array:
- * @caps: a #GstCaps
+ * @caps: (transfer full): a #GstCaps
  * @field: field in caps to set
  * @buf: header buffers
  *
  * Adds given buffers to an array of buffers set as the given @field
  * on the given @caps.  List of buffer arguments must be NULL-terminated.
  *
- * Returns: input caps with a streamheader field added, or NULL if some error
+ * Returns: (transfer full): input caps with a streamheader field added, or NULL
+ *     if some error occurred
  */
 static GstCaps *
 _gst_caps_set_buffer_array (GstCaps * caps, const gchar * field,
@@ -799,6 +800,27 @@ gst_vorbis_enc_output_buffers (GstVorbisEnc * vorbisenc)
     while (vorbis_bitrate_flushpacket (&vorbisenc->vd, &op)) {
       GstBuffer *buf;
 
+      if (op.e_o_s) {
+        GstAudioEncoder *enc = GST_AUDIO_ENCODER (vorbisenc);
+        GstClockTime duration;
+
+        GST_DEBUG_OBJECT (vorbisenc, "Got EOS packet from libvorbis");
+        GST_AUDIO_ENCODER_STREAM_LOCK (enc);
+        if (!GST_CLOCK_TIME_IS_VALID (enc->output_segment.stop)) {
+          GST_DEBUG_OBJECT (vorbisenc,
+              "Output segment has no end time, setting");
+          duration =
+              gst_util_uint64_scale (op.granulepos, GST_SECOND,
+              vorbisenc->frequency);
+          enc->output_segment.stop = enc->output_segment.start + duration;
+          GST_DEBUG_OBJECT (enc, "new output segment %" GST_SEGMENT_FORMAT,
+              &enc->output_segment);
+          gst_pad_push_event (GST_AUDIO_ENCODER_SRC_PAD (enc),
+              gst_event_new_segment (&enc->output_segment));
+        }
+        GST_AUDIO_ENCODER_STREAM_UNLOCK (enc);
+      }
+
       GST_LOG_OBJECT (vorbisenc, "pushing out a data packet");
       buf =
           gst_audio_encoder_allocate_output_buffer (GST_AUDIO_ENCODER
