@@ -67,33 +67,24 @@ struct _GstRTPBaseDepayload
  * GstRTPBaseDepayloadClass:
  * @parent_class: the parent class
  * @set_caps: configure the depayloader
- * @process: process incoming rtp packets. Subclass must implement either
- *   this method or @process_rtp_packet to process incoming rtp packets.
- *   If the child returns a buffer without a valid timestamp, the timestamp
- *   of the provided buffer will be applied to the result buffer and the
- *   buffer will be pushed. If this function returns %NULL, nothing is pushed.
+ * @process: process incoming rtp packets
  * @packet_lost: signal the depayloader about packet loss
  * @handle_event: custom event handling
- * @process_rtp_packet: Same as the process virtual function, but slightly more
- * efficient, since it is passed the rtp buffer structure that has already
- * been mapped (with GST_MAP_READ) by the base class and thus does not have
- * to be mapped again by the subclass. Can be used by the subclass to process
- * incoming rtp packets. If the subclass returns a buffer without a valid
- * timestamp, the timestamp of the input buffer will be applied to the result
- * buffer and the output buffer will be pushed out. If this function returns
- * %NULL, nothing is pushed out. Since: 1.6.
  *
- * Base class for RTP depayloaders.
+ * Base class for audio RTP payloader.
  */
 struct _GstRTPBaseDepayloadClass
 {
   GstElementClass parent_class;
 
-  /*< public >*/
   /* virtuals, inform the subclass of the caps. */
   gboolean (*set_caps) (GstRTPBaseDepayload *filter, GstCaps *caps);
 
-  /* pure virtual function */
+  /* pure virtual function, child must use this to process incoming
+   * rtp packets. If the child returns a buffer without a valid timestamp,
+   * the timestamp of @in will be applied to the result buffer and the
+   * buffer will be pushed. If this function returns %NULL, nothing is
+   * pushed.  */
   GstBuffer * (*process) (GstRTPBaseDepayload *base, GstBuffer *in);
 
   /* non-pure function used to to signal the depayloader about packet loss. the
@@ -105,30 +96,15 @@ struct _GstRTPBaseDepayloadClass
    * implementation can override. */
   gboolean (*handle_event) (GstRTPBaseDepayload * filter, GstEvent * event);
 
-  GstBuffer * (*process_rtp_packet) (GstRTPBaseDepayload *base, GstRTPBuffer * rtp_buffer);
-
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING - 1];
+  gpointer _gst_reserved[GST_PADDING];
 };
 
-GST_RTP_API
 GType gst_rtp_base_depayload_get_type (void);
 
-GST_RTP_API
 GstFlowReturn   gst_rtp_base_depayload_push       (GstRTPBaseDepayload *filter, GstBuffer *out_buf);
-
-GST_RTP_API
 GstFlowReturn   gst_rtp_base_depayload_push_list  (GstRTPBaseDepayload *filter, GstBufferList *out_list);
 
-GST_RTP_API
-gboolean        gst_rtp_base_depayload_is_source_info_enabled  (GstRTPBaseDepayload * depayload);
-
-GST_RTP_API
-void            gst_rtp_base_depayload_set_source_info_enabled (GstRTPBaseDepayload * depayload,
-                                                                gboolean enable);
-
-
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstRTPBaseDepayload, gst_object_unref)
 
 G_END_DECLS
 
