@@ -30,8 +30,6 @@
  * 'application/x-rtp' #GstCaps.
  * </para>
  * </refsect2>
- *
- * Last reviewed on 2006-07-17 (0.10.10)
  */
 
 #include "gstrtpbuffer.h"
@@ -296,6 +294,10 @@ gst_rtp_buffer_calc_payload_len (guint packet_len, guint8 pad_len,
 {
   g_return_val_if_fail (csrc_count <= 15, 0);
 
+  if (packet_len <
+      GST_RTP_HEADER_LEN + (csrc_count * sizeof (guint32)) + pad_len)
+    return 0;
+
   return packet_len - GST_RTP_HEADER_LEN - (csrc_count * sizeof (guint32))
       - pad_len;
 }
@@ -321,12 +323,14 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
   guint size;
   gsize bufsize, skip;
   guint idx, length;
+  guint n_mem;
 
   g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
   g_return_val_if_fail (rtp != NULL, FALSE);
   g_return_val_if_fail (rtp->buffer == NULL, FALSE);
 
-  if (gst_buffer_n_memory (buffer) < 1)
+  n_mem = gst_buffer_n_memory (buffer);
+  if (n_mem < 1)
     goto no_memory;
 
   /* map first memory, this should be the header */
@@ -405,11 +409,11 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
       goto map_failed;
 
     padding = rtp->map[3].data[skip];
-    if (skip + 1 < padding)
-      goto wrong_length;
-
     rtp->data[3] = rtp->map[3].data + skip + 1 - padding;
     rtp->size[3] = padding;
+
+    if (skip + 1 < padding)
+      goto wrong_length;
   } else {
     rtp->data[3] = NULL;
     rtp->size[3] = 0;
@@ -421,10 +425,19 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
     goto wrong_padding;
 
   rtp->buffer = buffer;
-  /* we have not yet mapped the payload */
-  rtp->data[2] = NULL;
-  rtp->size[2] = 0;
-  rtp->state = 0;
+
+  if (n_mem == 1) {
+    /* we have mapped the buffer already, so might just as well fill in the
+     * payload pointer and size and avoid another buffer map/unmap later */
+    rtp->data[2] = rtp->map[0].data + header_len;
+    rtp->size[2] = bufsize - header_len - padding;
+  } else {
+    /* we have not yet mapped the payload */
+    rtp->data[2] = NULL;
+    rtp->size[2] = 0;
+  }
+
+  /* rtp->state = 0; *//* unused */
 
   return TRUE;
 
@@ -467,7 +480,7 @@ dump_packet:
     GST_MEMDUMP ("buffer", data, size);
 
     for (i = 0; i < G_N_ELEMENTS (rtp->map); ++i) {
-      if (rtp->data[i] != NULL)
+      if (rtp->map[i].memory != NULL)
         gst_buffer_unmap (buffer, &rtp->map[i]);
     }
     return FALSE;
@@ -489,8 +502,12 @@ gst_rtp_buffer_unmap (GstRTPBuffer * rtp)
   g_return_if_fail (rtp->buffer != NULL);
 
   for (i = 0; i < 4; i++) {
-    if (rtp->data[i])
+    if (rtp->map[i].memory != NULL) {
       gst_buffer_unmap (rtp->buffer, &rtp->map[i]);
+      rtp->map[i].memory = NULL;
+    }
+    rtp->data[i] = NULL;
+    rtp->size[i] = 0;
   }
   rtp->buffer = NULL;
 }
@@ -701,7 +718,7 @@ gst_rtp_buffer_get_extension_data (GstRTPBuffer * rtp, guint16 * bits,
 }
 
 /**
- * gst_rtp_buffer_get_extension_bytes:
+ * gst_rtp_buffer_get_extension_bytes: (rename-to gst_rtp_buffer_get_extension_data)
  * @rtp: the RTP packet
  * @bits: (out): location for header bits
  *
@@ -717,8 +734,6 @@ gst_rtp_buffer_get_extension_data (GstRTPBuffer * rtp, guint16 * bits,
  * Returns: (transfer full): A new #GBytes if an extension header was present
  * and %NULL otherwise.
  *
- * Rename to: gst_rtp_buffer_get_extension_data
- *
  * Since: 1.2
  */
 GBytes *
@@ -1044,7 +1059,7 @@ gst_rtp_buffer_set_timestamp (GstRTPBuffer * rtp, guint32 timestamp)
  *
  * Create a subbuffer of the payload of the RTP packet in @buffer. @offset bytes
  * are skipped in the payload and the subbuffer will be of size @len.
- * If @len is -1 the total payload starting from @offset if subbuffered.
+ * If @len is -1 the total payload starting from @offset is subbuffered.
  *
  * Returns: A new buffer with the specified data of the payload.
  */
@@ -1056,7 +1071,7 @@ gst_rtp_buffer_get_payload_subbuffer (GstRTPBuffer * rtp, guint offset,
 
   plen = gst_rtp_buffer_get_payload_len (rtp);
   /* we can't go past the length */
-  if (G_UNLIKELY (offset >= plen))
+  if (G_UNLIKELY (offset > plen))
     goto wrong_offset;
 
   /* apply offset */
@@ -1073,7 +1088,7 @@ gst_rtp_buffer_get_payload_subbuffer (GstRTPBuffer * rtp, guint offset,
   /* ERRORS */
 wrong_offset:
   {
-    g_warning ("offset=%u should be less then plen=%u", offset, plen);
+    g_warning ("offset=%u should be less than plen=%u", offset, plen);
     return NULL;
   }
 }
@@ -1146,7 +1161,7 @@ gst_rtp_buffer_get_payload (GstRTPBuffer * rtp)
 }
 
 /**
- * gst_rtp_buffer_get_payload_bytes:
+ * gst_rtp_buffer_get_payload_bytes: (rename-to gst_rtp_buffer_get_payload)
  * @rtp: the RTP packet
  *
  * Similar to gst_rtp_buffer_get_payload, but more suitable for language
@@ -1155,8 +1170,6 @@ gst_rtp_buffer_get_payload (GstRTPBuffer * rtp)
  *
  * Returns: (transfer full): A new #GBytes containing the payload data in @rtp.
  *
- * Rename to: gst_rtp_buffer_get_payload
- *
  * Since: 1.2
  */
 GBytes *
@@ -1214,6 +1227,11 @@ gst_rtp_buffer_default_clock_rate (guint8 payload_type)
 gint
 gst_rtp_buffer_compare_seqnum (guint16 seqnum1, guint16 seqnum2)
 {
+  /* See http://en.wikipedia.org/wiki/Serial_number_arithmetic
+   * for an explanation why this does the right thing even for
+   * wraparounds, under the assumption that the difference is
+   * never bigger than 2**15 sequence numbers
+   */
   return (gint16) (seqnum2 - seqnum1);
 }
 
@@ -1243,7 +1261,7 @@ gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
     result = timestamp;
   } else {
     /* pick wraparound counter from previous timestamp and add to new timestamp */
-    result = timestamp + (ext & ~(G_GINT64_CONSTANT (0xffffffff)));
+    result = timestamp + (ext & ~(G_GUINT64_CONSTANT (0xffffffff)));
 
     /* check for timestamp wraparound */
     if (result < ext)
@@ -1254,7 +1272,7 @@ gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
     if (diff > G_MAXINT32) {
       /* timestamp went backwards more than allowed, we wrap around and get
        * updated extended timestamp. */
-      result += (G_GINT64_CONSTANT (1) << 32);
+      result += (G_GUINT64_CONSTANT (1) << 32);
     }
   }
   *exttimestamp = result;
@@ -1518,7 +1536,7 @@ gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
 
 
 static guint
-get_twobytes_header_end_offset (guint8 * pdata, guint wordlen)
+get_twobytes_header_end_offset (const guint8 * pdata, guint wordlen)
 {
   guint offset = 0;
   guint bytelen = wordlen * 4;
