@@ -96,6 +96,38 @@ gst_video_meta_get_info (void)
   return video_meta_info;
 }
 
+/**
+ * gst_buffer_get_video_meta:
+ * @buffer: a #GstBuffer
+ *
+ * Find the #GstVideoMeta on @buffer with the lowest @id.
+ *
+ * Buffers can contain multiple #GstVideoMeta metadata items when dealing with
+ * multiview buffers.
+ *
+ * Returns: the #GstVideoMeta with lowest id (usually 0) or %NULL when there
+ * is no such metadata on @buffer.
+ */
+GstVideoMeta *
+gst_buffer_get_video_meta (GstBuffer * buffer)
+{
+  gpointer state = NULL;
+  GstVideoMeta *out = NULL;
+  GstMeta *meta;
+  const GstMetaInfo *info = GST_VIDEO_META_INFO;
+
+  while ((meta = gst_buffer_iterate_meta (buffer, &state))) {
+    if (meta->info->api == info->api) {
+      GstVideoMeta *vmeta = (GstVideoMeta *) meta;
+      if (vmeta->id == 0)
+        return vmeta;           /* Early out for id 0 */
+      if (out == NULL || vmeta->id < out->id)
+        out = vmeta;
+    }
+  }
+  return out;
+}
+
 /**
  * gst_buffer_get_video_meta_id:
  * @buffer: a #GstBuffer
@@ -443,7 +475,7 @@ gst_video_gl_texture_upload_meta_transform (GstBuffer * dest, GstMeta * meta,
       dmeta->texture_orientation = smeta->texture_orientation;
       dmeta->n_textures = smeta->n_textures;
       memcpy (dmeta->texture_type, smeta->texture_type,
-          sizeof (smeta->texture_type[0] * 4));
+          sizeof (smeta->texture_type[0]) * 4);
       dmeta->buffer = dest;
       dmeta->upload = smeta->upload;
       dmeta->user_data = smeta->user_data;
@@ -509,7 +541,7 @@ gst_buffer_add_video_gl_texture_upload_meta (GstBuffer * buffer,
 
   meta->texture_orientation = texture_orientation;
   meta->n_textures = n_textures;
-  memcpy (meta->texture_type, texture_type, sizeof (texture_type[0] * 4));
+  memcpy (meta->texture_type, texture_type, sizeof (texture_type[0]) * 4);
   meta->buffer = buffer;
   meta->upload = upload;
   meta->user_data = user_data;
@@ -645,7 +677,7 @@ gst_video_region_of_interest_meta_get_info (void)
  * Buffers can contain multiple #GstVideoRegionOfInterestMeta metadata items if
  * multiple regions of interests are marked on a frame.
  *
- * Returns: the #GstVideoeRegionOfInterestMeta with @id or %NULL when there is
+ * Returns: the #GstVideoRegionOfInterestMeta with @id or %NULL when there is
  * no such metadata on @buffer.
  */
 GstVideoRegionOfInterestMeta *
