@@ -16,20 +16,12 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "gst/video/gstvideometa.h"
 #include "gst/video/gstvideopool.h"
 
-
-GST_DEBUG_CATEGORY_STATIC (gst_video_pool_debug);
-#define GST_CAT_DEFAULT gst_video_pool_debug
-
 /**
  * SECTION:gstvideopool
- * @title: GstVideoBufferPool
  * @short_description: GstBufferPool for raw video buffers
  * @see_also: #GstBufferPool
  *
@@ -50,7 +42,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_video_pool_debug);
  */
 void
 gst_buffer_pool_config_set_video_alignment (GstStructure * config,
-    const GstVideoAlignment * align)
+    GstVideoAlignment * align)
 {
   g_return_if_fail (config != NULL);
   g_return_if_fail (align != NULL);
@@ -74,10 +66,10 @@ gst_buffer_pool_config_set_video_alignment (GstStructure * config,
  * Get the video alignment from the bufferpool configuration @config in
  * in @align
  *
- * Returns: %TRUE if @config could be parsed correctly.
+ * Returns: #TRUE if @config could be parsed correctly.
  */
 gboolean
-gst_buffer_pool_config_get_video_alignment (const GstStructure * config,
+gst_buffer_pool_config_get_video_alignment (GstStructure * config,
     GstVideoAlignment * align)
 {
   g_return_val_if_fail (config != NULL, FALSE);
@@ -97,6 +89,7 @@ gst_buffer_pool_config_get_video_alignment (const GstStructure * config,
 /* bufferpool */
 struct _GstVideoBufferPoolPrivate
 {
+  GstCaps *caps;
   GstVideoInfo info;
   GstVideoAlignment video_align;
   gboolean add_videometa;
@@ -107,9 +100,11 @@ struct _GstVideoBufferPoolPrivate
 
 static void gst_video_buffer_pool_finalize (GObject * object);
 
+#define GST_VIDEO_BUFFER_POOL_GET_PRIVATE(obj)  \
+   (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_VIDEO_BUFFER_POOL, GstVideoBufferPoolPrivate))
+
 #define gst_video_buffer_pool_parent_class parent_class
-G_DEFINE_TYPE_WITH_PRIVATE (GstVideoBufferPool, gst_video_buffer_pool,
-    GST_TYPE_BUFFER_POOL);
+G_DEFINE_TYPE (GstVideoBufferPool, gst_video_buffer_pool, GST_TYPE_BUFFER_POOL);
 
 static const gchar **
 video_buffer_pool_get_options (GstBufferPool * pool)
@@ -140,19 +135,8 @@ video_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
     goto no_caps;
 
   /* now parse the caps from the config */
-  if (gst_video_is_dma_drm_caps (caps)) {
-    GstVideoInfoDmaDrm drm_info;
-    if (!gst_video_info_dma_drm_from_caps (&drm_info, caps))
-      goto wrong_caps;
-
-    if (GST_VIDEO_INFO_FORMAT (&drm_info.vinfo) == GST_VIDEO_FORMAT_DMA_DRM)
-      goto wrong_caps;
-
-    info = drm_info.vinfo;
-  } else {
-    if (!gst_video_info_from_caps (&info, caps))
-      goto wrong_caps;
-  }
+  if (!gst_video_info_from_caps (&info, caps))
+    goto wrong_caps;
 
   if (size < info.size)
     goto wrong_size;
@@ -165,6 +149,10 @@ video_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
 
   GST_LOG_OBJECT (pool, "%dx%d, caps %" GST_PTR_FORMAT, width, height, caps);
 
+  if (priv->caps)
+    gst_caps_unref (priv->caps);
+  priv->caps = gst_caps_ref (caps);
+
   priv->params = params;
   if (priv->allocator)
     gst_object_unref (priv->allocator);
@@ -181,34 +169,13 @@ video_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
       GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT);
 
   if (priv->need_alignment && priv->add_videometa) {
-    guint max_align, n;
-
+    /* get an apply the alignment to the info */
     gst_buffer_pool_config_get_video_alignment (config, &priv->video_align);
-
-    /* ensure GstAllocationParams alignment is compatible with video alignment */
-    max_align = priv->params.align;
-    for (n = 0; n < GST_VIDEO_MAX_PLANES; ++n)
-      max_align |= priv->video_align.stride_align[n];
-
-    for (n = 0; n < GST_VIDEO_MAX_PLANES; ++n)
-      priv->video_align.stride_align[n] = max_align;
-
-    /* apply the alignment to the info */
-    if (!gst_video_info_align (&info, &priv->video_align))
-      goto failed_to_align;
-
+    gst_video_info_align (&info, &priv->video_align);
     gst_buffer_pool_config_set_video_alignment (config, &priv->video_align);
-
-    if (priv->params.align < max_align) {
-      GST_WARNING_OBJECT (pool, "allocation params alignment %u is smaller "
-          "than the max specified video stride alignment %u, fixing",
-          (guint) priv->params.align, max_align);
-      priv->params.align = max_align;
-      gst_buffer_pool_config_set_allocator (config, allocator, &priv->params);
-    }
   }
-  info.size = MAX (size, info.size);
   priv->info = info;
+  info.size = MAX (size, info.size);
 
   gst_buffer_pool_config_set_params (config, caps, info.size, min_buffers,
       max_buffers);
@@ -235,16 +202,9 @@ wrong_caps:
 wrong_size:
   {
     GST_WARNING_OBJECT (pool,
-        "Provided size is to small for the caps: %u < %" G_GSIZE_FORMAT, size,
-        info.size);
-    gst_buffer_pool_config_set_params (config, caps, info.size, min_buffers,
-        max_buffers);
-    return FALSE;
-  }
-failed_to_align:
-  {
-    GST_WARNING_OBJECT (pool, "Failed to align");
+        "Provided size is to small for the caps: %u", size);
     return FALSE;
+
   }
 }
 
@@ -290,15 +250,14 @@ no_memory:
  * Create a new bufferpool that can allocate video frames. This bufferpool
  * supports all the video bufferpool options.
  *
- * Returns: (transfer full): a new #GstBufferPool to allocate video frames
+ * Returns: (transfer floating): a new #GstBufferPool to allocate video frames
  */
 GstBufferPool *
-gst_video_buffer_pool_new (void)
+gst_video_buffer_pool_new ()
 {
   GstVideoBufferPool *pool;
 
   pool = g_object_new (GST_TYPE_VIDEO_BUFFER_POOL, NULL);
-  gst_object_ref_sink (pool);
 
   GST_LOG_OBJECT (pool, "new video buffer pool %p", pool);
 
@@ -311,20 +270,19 @@ gst_video_buffer_pool_class_init (GstVideoBufferPoolClass * klass)
   GObjectClass *gobject_class = (GObjectClass *) klass;
   GstBufferPoolClass *gstbufferpool_class = (GstBufferPoolClass *) klass;
 
+  g_type_class_add_private (klass, sizeof (GstVideoBufferPoolPrivate));
+
   gobject_class->finalize = gst_video_buffer_pool_finalize;
 
   gstbufferpool_class->get_options = video_buffer_pool_get_options;
   gstbufferpool_class->set_config = video_buffer_pool_set_config;
   gstbufferpool_class->alloc_buffer = video_buffer_pool_alloc;
-
-  GST_DEBUG_CATEGORY_INIT (gst_video_pool_debug, "videopool", 0,
-      "videopool object");
 }
 
 static void
 gst_video_buffer_pool_init (GstVideoBufferPool * pool)
 {
-  pool->priv = gst_video_buffer_pool_get_instance_private (pool);
+  pool->priv = GST_VIDEO_BUFFER_POOL_GET_PRIVATE (pool);
 }
 
 static void
@@ -335,6 +293,9 @@ gst_video_buffer_pool_finalize (GObject * object)
 
   GST_LOG_OBJECT (pool, "finalize video buffer pool %p", pool);
 
+  if (priv->caps)
+    gst_caps_unref (priv->caps);
+
   if (priv->allocator)
     gst_object_unref (priv->allocator);
 
