@@ -16,9 +16,6 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include <string.h>
 
@@ -27,7 +24,6 @@
 
 /**
  * SECTION:gstvideodither
- * @title: GstVideoDither
  * @short_description: Utility object for dithering and quantizing lines of video
  *
  * GstVideoDither provides implementations of several dithering algorithms
@@ -355,18 +351,18 @@ count_power (guint v)
 }
 
 /**
- * gst_video_dither_new: (constructor) (skip)
+ * gst_video_dither_new:
  * @method: a #GstVideoDitherMethod
  * @flags: a #GstVideoDitherFlags
  * @format: a #GstVideoFormat
- * @quantizer: (array fixed-size=4): quantizer
+ * @quantizer: quantizer
  * @width: the width of the lines
  *
  * Make a new dither object for dithering lines of @format using the
  * algorithm described by @method.
  *
  * Each component will be quantized to a multiple of @quantizer. Better
- * performance is achieved when @quantizer is a power of 2.
+ * performance is achived when @quantizer is a power of 2.
  *
  * @width is the width of the lines that this ditherer will handle.
  *
@@ -380,7 +376,7 @@ gst_video_dither_new (GstVideoDitherMethod method, GstVideoDitherFlags flags,
   GstVideoDither *dither;
   gint i;
 
-  dither = g_new0 (GstVideoDither, 1);
+  dither = g_slice_new0 (GstVideoDither);
   dither->method = method;
   dither->flags = flags;
   dither->format = format;
@@ -398,7 +394,7 @@ gst_video_dither_new (GstVideoDitherMethod method, GstVideoDitherFlags flags,
       dither->depth = 16;
       break;
     default:
-      g_free (dither);
+      g_slice_free (GstVideoDither, dither);
       g_return_val_if_reached (NULL);
       break;
   }
@@ -417,7 +413,7 @@ gst_video_dither_new (GstVideoDitherMethod method, GstVideoDitherFlags flags,
   }
   dither->orc_mask64 = GUINT64_FROM_BE (dither->orc_mask64);
   dither->orc_mask32 = GUINT32_FROM_BE (dither->orc_mask32);
-  GST_DEBUG ("mask64 %08" G_GINT64_MODIFIER "x", (guint64) dither->orc_mask64);
+  GST_DEBUG ("mask64 %08llx", (unsigned long long int) dither->orc_mask64);
   GST_DEBUG ("mask32 %08x", dither->orc_mask32);
 
   switch (method) {
@@ -470,7 +466,7 @@ gst_video_dither_free (GstVideoDither * dither)
   g_return_if_fail (dither != NULL);
 
   g_free (dither->errors);
-  g_free (dither);
+  g_slice_free (GstVideoDither, dither);
 }
 
 /**
