@@ -21,32 +21,39 @@
 
 /**
  * SECTION:gstvideooverlaycomposition
- * @title: GstVideoOverlayRectangle
  * @short_description: Video Buffer Overlay Compositions (Subtitles, Logos)
  *
+ * <refsect2>
+ * <para>
  * Functions to create and handle overlay compositions on video buffers.
- *
+ * </para>
+ * <para>
  * An overlay composition describes one or more overlay rectangles to be
  * blended on top of a video buffer.
- *
+ * </para>
+ * <para>
  * This API serves two main purposes:
- *
- * * it can be used to attach overlay information (subtitles or logos)
- *   to non-raw video buffers such as GL/VAAPI/VDPAU surfaces. The actual
- *   blending of the overlay can then be done by e.g. the video sink that
- *   processes these non-raw buffers.
- *
- * * it can also be used to blend overlay rectangles on top of raw video
- *   buffers, thus consolidating blending functionality for raw video in
- *   one place.
- *
+ * <itemizedlist>
+ * <listitem>
+ * it can be used to attach overlay information (subtitles or logos)
+ * to non-raw video buffers such as GL/VAAPI/VDPAU surfaces. The actual
+ * blending of the overlay can then be done by e.g. the video sink that
+ * processes these non-raw buffers.
+ * </listitem>
+ * <listitem>
+ * it can also be used to blend overlay rectangles on top of raw video
+ * buffers, thus consolidating blending functionality for raw video in
+ * one place.
+ * </listitem>
  * Together, this allows existing overlay elements to easily handle raw
  * and non-raw video as input in without major changes (once the overlays
- * have been put into a #GstVideoOverlayComposition object anyway) - for raw
+ * have been put into a #GstOverlayComposition object anyway) - for raw
  * video the overlay can just use the blending function to blend the data
  * on top of the video, and for surface buffers it can just attach them to
  * the buffer and let the sink render the overlays.
- *
+ * </itemizedlist>
+ * </para>
+ * </refsect2>
  */
 
 /* TODO:
@@ -179,19 +186,6 @@ gst_video_overlay_get_seqnum (void)
   return (guint) g_atomic_int_add (&seqnum, 1);
 }
 
-static gboolean
-gst_video_overlay_composition_meta_init (GstMeta * meta, gpointer params,
-    GstBuffer * buf)
-{
-  GstVideoOverlayCompositionMeta *ometa;
-
-  ometa = (GstVideoOverlayCompositionMeta *) meta;
-
-  ometa->overlay = NULL;
-
-  return TRUE;
-}
-
 static void
 gst_video_overlay_composition_meta_free (GstMeta * meta, GstBuffer * buf)
 {
@@ -207,118 +201,22 @@ static gboolean
 gst_video_overlay_composition_meta_transform (GstBuffer * dest, GstMeta * meta,
     GstBuffer * buffer, GQuark type, gpointer data)
 {
-  GstVideoOverlayCompositionMeta *dmeta = NULL, *smeta;
+  GstVideoOverlayCompositionMeta *dmeta, *smeta;
 
   smeta = (GstVideoOverlayCompositionMeta *) meta;
 
   if (GST_META_TRANSFORM_IS_COPY (type)) {
-    GST_DEBUG ("copy video overlay composition metadata");
+    GstMetaTransformCopy *copy = data;
 
-    dmeta =
-        (GstVideoOverlayCompositionMeta *) gst_buffer_add_meta (dest,
-        GST_VIDEO_OVERLAY_COMPOSITION_META_INFO, NULL);
-    if (!dmeta)
-      return FALSE;
+    if (!copy->region) {
+      GST_DEBUG ("copy video overlay composition metadata");
 
-    dmeta->overlay = gst_video_overlay_composition_ref (smeta->overlay);
-  } else if (GST_VIDEO_META_TRANSFORM_IS_MATRIX (type)) {
-    GstVideoMetaTransformMatrix *trans = data;
-    GstVideoOverlayComposition *new_comp;
-    guint n_rectangles;
-    gboolean have_rect = FALSE;
-
-    new_comp = gst_video_overlay_composition_new (NULL);
-    n_rectangles = gst_video_overlay_composition_n_rectangles (smeta->overlay);
-    for (guint i = 0; i < n_rectangles; i++) {
-      GstVideoOverlayRectangle *rect =
-          gst_video_overlay_composition_get_rectangle (smeta->overlay, i);
-      GstVideoOverlayRectangle *new_rect =
-          gst_video_overlay_rectangle_copy (rect);
-      GstVideoRectangle render_rect;
-      guint width = G_MAXINT;
-      guint height = G_MAXINT;
-
-      gst_video_overlay_rectangle_get_render_rectangle (rect, &render_rect.x,
-          &render_rect.y, &width, &height);
-      render_rect.w = MIN (width, G_MAXINT);
-      render_rect.h = MIN (height, G_MAXINT);
-
-      GstVideoRectangle orig_render_rect = render_rect;
-
-      if (gst_video_meta_transform_matrix_rectangle (trans, &render_rect)) {
-
-        GST_DEBUG ("overlay rectangle %u (seqnum: %d) (%dx%d)x(%dx%d)"
-            " -> (%dx%d)->(%dx%d)", i,
-            gst_video_overlay_rectangle_get_seqnum (rect), orig_render_rect.x,
-            orig_render_rect.y, orig_render_rect.w, orig_render_rect.h,
-            render_rect.x, render_rect.y, render_rect.w, render_rect.h);
-
-        gst_video_overlay_rectangle_set_render_rectangle (new_rect,
-            render_rect.x, render_rect.y, render_rect.w, render_rect.h);
-        gst_video_overlay_composition_add_rectangle (new_comp, new_rect);
-        have_rect = TRUE;
-      }
-      gst_video_overlay_rectangle_unref (new_rect);
+      /* only copy if the complete data is copied as well */
+      dmeta =
+          (GstVideoOverlayCompositionMeta *) gst_buffer_add_meta (dest,
+          GST_VIDEO_OVERLAY_COMPOSITION_META_INFO, NULL);
+      dmeta->overlay = gst_video_overlay_composition_ref (smeta->overlay);
     }
-
-    if (have_rect)
-      dmeta = gst_buffer_add_video_overlay_composition_meta (dest, new_comp);
-    gst_video_overlay_composition_unref (new_comp);
-    if (!have_rect || !dmeta)
-      return FALSE;
-  } else if (GST_VIDEO_META_TRANSFORM_IS_SCALE (type)) {
-    GstVideoMetaTransform *trans = data;
-    GstVideoOverlayComposition *new_comp;
-    gint ow, oh, nw, nh;
-    guint n_rectangles;
-
-    ow = GST_VIDEO_INFO_WIDTH (trans->in_info);
-    nw = GST_VIDEO_INFO_WIDTH (trans->out_info);
-    oh = GST_VIDEO_INFO_HEIGHT (trans->in_info);
-    nh = GST_VIDEO_INFO_HEIGHT (trans->out_info);
-
-    GST_DEBUG ("scaling video overlay composition metadata %dx%d -> %dx%d", ow,
-        oh, nw, nh);
-
-    new_comp = gst_video_overlay_composition_new (NULL);
-    n_rectangles = gst_video_overlay_composition_n_rectangles (smeta->overlay);
-    for (guint i = 0; i < n_rectangles; i++) {
-      GstVideoOverlayRectangle *rect =
-          gst_video_overlay_composition_get_rectangle (smeta->overlay, i);
-      GstVideoOverlayRectangle *new_rect =
-          gst_video_overlay_rectangle_copy (rect);
-      gint render_x = 0, render_y = 0;
-      gint new_render_x, new_render_y;
-      guint render_width = 0, render_height = 0;
-      guint new_render_width, new_render_height;
-
-      gst_video_overlay_rectangle_get_render_rectangle (rect, &render_x,
-          &render_y, &render_width, &render_height);
-
-      new_render_x = (render_x * nw) / ow;
-      new_render_y = (render_y * nh) / oh;
-      new_render_width = (render_width * nw) / ow;
-      new_render_height = (render_height * nh) / oh;
-
-      GST_DEBUG
-          ("overlay rectangle %u (seqnum: %d) (%dx%d)x(%ux%u) -> (%dx%d)->(%ux%u)",
-          i, gst_video_overlay_rectangle_get_seqnum (rect), render_x, render_y,
-          render_width, render_height, new_render_x, new_render_y,
-          new_render_width, new_render_height);
-
-      gst_video_overlay_rectangle_set_render_rectangle (new_rect,
-          new_render_x, new_render_y, new_render_width, new_render_height);
-      gst_video_overlay_composition_add_rectangle (new_comp, new_rect);
-      gst_video_overlay_rectangle_unref (new_rect);
-    }
-
-    dmeta = gst_buffer_add_video_overlay_composition_meta (dest, new_comp);
-    gst_video_overlay_composition_unref (new_comp);
-    if (!dmeta)
-      return FALSE;
-  } else {
-    /* return FALSE, if transform type is not supported */
-    return FALSE;
   }
   return TRUE;
 }
@@ -326,11 +224,8 @@ gst_video_overlay_composition_meta_transform (GstBuffer * dest, GstMeta * meta,
 GType
 gst_video_overlay_composition_meta_api_get_type (void)
 {
-  static GType type = 0;
-  static const gchar *tags[] =
-      { GST_META_TAG_VIDEO_STR, GST_META_TAG_VIDEO_ORIENTATION_STR,
-    GST_META_TAG_VIDEO_SIZE_STR, NULL
-  };
+  static volatile GType type = 0;
+  static const gchar *tags[] = { NULL };
 
   if (g_once_init_enter (&type)) {
     GType _type =
@@ -346,18 +241,15 @@ gst_video_overlay_composition_meta_get_info (void)
 {
   static const GstMetaInfo *video_overlay_composition_meta_info = NULL;
 
-  if (g_once_init_enter ((GstMetaInfo **) &
-          video_overlay_composition_meta_info)) {
+  if (g_once_init_enter (&video_overlay_composition_meta_info)) {
     const GstMetaInfo *meta =
         gst_meta_register (GST_VIDEO_OVERLAY_COMPOSITION_META_API_TYPE,
         "GstVideoOverlayCompositionMeta",
-        sizeof (GstVideoOverlayCompositionMeta),
-        (GstMetaInitFunction) gst_video_overlay_composition_meta_init,
+        sizeof (GstVideoOverlayCompositionMeta), (GstMetaInitFunction) NULL,
         (GstMetaFreeFunction) gst_video_overlay_composition_meta_free,
         (GstMetaTransformFunction)
         gst_video_overlay_composition_meta_transform);
-    g_once_init_leave ((GstMetaInfo **) & video_overlay_composition_meta_info,
-        (GstMetaInfo *) meta);
+    g_once_init_leave (&video_overlay_composition_meta_info, meta);
   }
   return video_overlay_composition_meta_info;
 }
@@ -370,8 +262,6 @@ gst_video_overlay_composition_meta_get_info (void)
  * Sets an overlay composition on a buffer. The buffer will obtain its own
  * reference to the composition, meaning this function does not take ownership
  * of @comp.
- *
- * Returns: (transfer none): a #GstVideoOverlayCompositionMeta
  */
 GstVideoOverlayCompositionMeta *
 gst_buffer_add_video_overlay_composition_meta (GstBuffer * buf,
@@ -405,8 +295,6 @@ gst_video_overlay_composition_free (GstMiniObject * mini_obj)
   num = comp->num_rectangles;
 
   while (num > 0) {
-    gst_mini_object_remove_parent (GST_MINI_OBJECT_CAST (comp->rectangles[num -
-                1]), GST_MINI_OBJECT_CAST (comp));
     gst_video_overlay_rectangle_unref (comp->rectangles[num - 1]);
     --num;
   }
@@ -415,19 +303,17 @@ gst_video_overlay_composition_free (GstMiniObject * mini_obj)
   comp->rectangles = NULL;
   comp->num_rectangles = 0;
 
-  g_free (comp);
+  g_slice_free (GstVideoOverlayComposition, comp);
 }
 
 /**
  * gst_video_overlay_composition_new:
- * @rectangle: (transfer none) (nullable): a #GstVideoOverlayRectangle to add to the
+ * @rectangle: (transfer none): a #GstVideoOverlayRectangle to add to the
  *     composition
  *
  * Creates a new video overlay composition object to hold one or more
  * overlay rectangles.
  *
- * Note that since 1.20 this allows to pass %NULL for @rectangle.
- *
  * Returns: (transfer full): a new #GstVideoOverlayComposition. Unref with
  *     gst_video_overlay_composition_unref() when no longer needed.
  */
@@ -436,10 +322,13 @@ gst_video_overlay_composition_new (GstVideoOverlayRectangle * rectangle)
 {
   GstVideoOverlayComposition *comp;
 
-  g_return_val_if_fail (GST_IS_VIDEO_OVERLAY_RECTANGLE (rectangle)
-      || rectangle == NULL, NULL);
 
-  comp = g_new0 (GstVideoOverlayComposition, 1);
+  /* FIXME: should we allow empty compositions? Could also be expressed as
+   * buffer without a composition on it. Maybe there are cases where doing
+   * an empty new + _add() in a loop is easier? */
+  g_return_val_if_fail (GST_IS_VIDEO_OVERLAY_RECTANGLE (rectangle), NULL);
+
+  comp = g_slice_new0 (GstVideoOverlayComposition);
 
   gst_mini_object_init (GST_MINI_OBJECT_CAST (comp), 0,
       GST_TYPE_VIDEO_OVERLAY_COMPOSITION,
@@ -447,17 +336,16 @@ gst_video_overlay_composition_new (GstVideoOverlayRectangle * rectangle)
       NULL, (GstMiniObjectFreeFunction) gst_video_overlay_composition_free);
 
   comp->rectangles = g_new0 (GstVideoOverlayRectangle *, RECTANGLE_ARRAY_STEP);
+  comp->rectangles[0] = gst_video_overlay_rectangle_ref (rectangle);
+  comp->num_rectangles = 1;
 
   comp->seq_num = gst_video_overlay_get_seqnum ();
-  comp->min_seq_num_used = comp->seq_num;
 
-  GST_LOG ("new composition %p: seq_num %u", comp, comp->seq_num);
+  /* since the rectangle was created earlier, its seqnum is smaller than ours */
+  comp->min_seq_num_used = rectangle->seq_num;
 
-  if (rectangle) {
-    /* since the rectangle was created earlier, its seqnum is smaller than ours */
-    comp->min_seq_num_used = rectangle->seq_num;
-    gst_video_overlay_composition_add_rectangle (comp, rectangle);
-  }
+  GST_LOG ("new composition %p: seq_num %u with rectangle %p", comp,
+      comp->seq_num, rectangle);
 
   return comp;
 }
@@ -477,7 +365,7 @@ gst_video_overlay_composition_add_rectangle (GstVideoOverlayComposition * comp,
 {
   g_return_if_fail (GST_IS_VIDEO_OVERLAY_COMPOSITION (comp));
   g_return_if_fail (GST_IS_VIDEO_OVERLAY_RECTANGLE (rectangle));
-  g_return_if_fail (gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (comp)));
+  g_return_if_fail (GST_MINI_OBJECT_REFCOUNT_VALUE (comp) == 1);
 
   if (comp->num_rectangles % RECTANGLE_ARRAY_STEP == 0) {
     comp->rectangles =
@@ -487,8 +375,6 @@ gst_video_overlay_composition_add_rectangle (GstVideoOverlayComposition * comp,
 
   comp->rectangles[comp->num_rectangles] =
       gst_video_overlay_rectangle_ref (rectangle);
-  gst_mini_object_add_parent (GST_MINI_OBJECT_CAST (rectangle),
-      GST_MINI_OBJECT_CAST (comp));
   comp->num_rectangles += 1;
 
   comp->min_seq_num_used = MIN (comp->min_seq_num_used, rectangle->seq_num);
@@ -519,7 +405,7 @@ gst_video_overlay_composition_n_rectangles (GstVideoOverlayComposition * comp)
  *
  * Returns the @n-th #GstVideoOverlayRectangle contained in @comp.
  *
- * Returns: (transfer none) (nullable): the @n-th rectangle, or NULL if @n is out of
+ * Returns: (transfer none): the @n-th rectangle, or NULL if @n is out of
  *     bounds. Will not return a new reference, the caller will need to
  *     obtain her own reference using gst_video_overlay_rectangle_ref()
  *     if needed.
@@ -546,15 +432,11 @@ gst_video_overlay_rectangle_needs_scaling (GstVideoOverlayRectangle * r)
 /**
  * gst_video_overlay_composition_blend:
  * @comp: a #GstVideoOverlayComposition
- * @video_buf: a #GstVideoFrame containing raw video data in a
- *             supported format. It should be mapped using GST_MAP_READWRITE
+ * @video_buf: a #GstVideoFrame containing raw video data in a supported format
  *
  * Blends the overlay rectangles in @comp on top of the raw video data
  * contained in @video_buf. The data in @video_buf must be writable and
  * mapped appropriately.
- *
- * Since @video_buf data is read and will be modified, it ought be
- * mapped with flag GST_MAP_READWRITE.
  */
 /* FIXME: formats with more than 8 bit per component which get unpacked into
  * ARGB64 or AYUV64 (such as v210, v216, UYVP, GRAY16_LE and GRAY16_BE)
@@ -630,11 +512,6 @@ gst_video_overlay_composition_blend (GstVideoOverlayComposition * comp,
  * actual overlay pixel data buffers contained in the rectangles are not
  * copied.
  *
- * This should be avoided unless rectangles need to be modified because it
- * invalidates caching in sinks and compositor elements. To add extra rectangles
- * it is rather recommended to add an extra composition meta using
- * gst_buffer_add_video_overlay_composition_meta().
- *
  * Returns: (transfer full): a new #GstVideoOverlayComposition equivalent
  *     to @comp.
  */
@@ -673,11 +550,6 @@ gst_video_overlay_composition_copy (GstVideoOverlayComposition * comp)
  * rectangles will also be copied, but the actual overlay pixel data buffers
  * contained in the rectangles are not copied.
  *
- * This should be avoided unless rectangles need to be modified because it
- * invalidates caching in sinks and compositor elements. To add extra rectangles
- * it is rather recommended to add an extra composition meta using
- * gst_buffer_add_video_overlay_composition_meta().
- *
  * Returns: (transfer full): a writable #GstVideoOverlayComposition
  *     equivalent to @comp.
  */
@@ -688,12 +560,11 @@ gst_video_overlay_composition_make_writable (GstVideoOverlayComposition * comp)
 
   g_return_val_if_fail (GST_IS_VIDEO_OVERLAY_COMPOSITION (comp), NULL);
 
-  if (gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (comp))) {
+  if (GST_MINI_OBJECT_REFCOUNT_VALUE (comp) == 1) {
     guint n;
 
     for (n = 0; n < comp->num_rectangles; ++n) {
-      if (!gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (comp->rectangles
-                  [n])))
+      if (GST_MINI_OBJECT_REFCOUNT_VALUE (comp->rectangles[n]) != 1)
         goto copy;
     }
     return comp;
@@ -736,8 +607,6 @@ gst_video_overlay_rectangle_free (GstMiniObject * mini_obj)
 {
   GstVideoOverlayRectangle *rect = (GstVideoOverlayRectangle *) mini_obj;
 
-  gst_mini_object_remove_parent (GST_MINI_OBJECT_CAST (rect->pixels),
-      GST_MINI_OBJECT_CAST (rect));
   gst_buffer_replace (&rect->pixels, NULL);
 
   while (rect->scaled_rectangles != NULL) {
@@ -752,10 +621,9 @@ gst_video_overlay_rectangle_free (GstMiniObject * mini_obj)
   g_free (rect->initial_alpha);
   g_mutex_clear (&rect->lock);
 
-  g_free (rect);
+  g_slice_free (GstVideoOverlayRectangle, rect);
 }
 
-#ifndef G_DISABLE_CHECKS
 static inline gboolean
 gst_video_overlay_rectangle_check_flags (GstVideoOverlayFormatFlags flags)
 {
@@ -763,7 +631,6 @@ gst_video_overlay_rectangle_check_flags (GstVideoOverlayFormatFlags flags)
   return (flags & ~(GST_VIDEO_OVERLAY_FORMAT_FLAG_PREMULTIPLIED_ALPHA |
           GST_VIDEO_OVERLAY_FORMAT_FLAG_GLOBAL_ALPHA)) == 0;
 }
-#endif
 
 static gboolean
 gst_video_overlay_rectangle_is_same_alpha_type (GstVideoOverlayFormatFlags
@@ -831,7 +698,7 @@ gst_video_overlay_rectangle_new_raw (GstBuffer * pixels,
       NULL);
   g_return_val_if_fail (height > 0 && width > 0, NULL);
 
-  rect = g_new0 (GstVideoOverlayRectangle, 1);
+  rect = g_slice_new0 (GstVideoOverlayRectangle);
 
   gst_mini_object_init (GST_MINI_OBJECT_CAST (rect), 0,
       GST_TYPE_VIDEO_OVERLAY_RECTANGLE,
@@ -841,15 +708,10 @@ gst_video_overlay_rectangle_new_raw (GstBuffer * pixels,
   g_mutex_init (&rect->lock);
 
   rect->pixels = gst_buffer_ref (pixels);
-  gst_mini_object_add_parent (GST_MINI_OBJECT_CAST (pixels),
-      GST_MINI_OBJECT_CAST (rect));
   rect->scaled_rectangles = NULL;
 
   gst_video_info_init (&rect->info);
-  if (!gst_video_info_set_format (&rect->info, format, width, height)) {
-    gst_mini_object_unref (GST_MINI_OBJECT_CAST (rect));
-    return NULL;
-  }
+  gst_video_info_set_format (&rect->info, format, width, height);
   if (flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_PREMULTIPLIED_ALPHA)
     rect->info.flags |= GST_VIDEO_FLAG_PREMULTIPLIED_ALPHA;
 
@@ -930,8 +792,7 @@ gst_video_overlay_rectangle_set_render_rectangle (GstVideoOverlayRectangle *
     guint render_height)
 {
   g_return_if_fail (GST_IS_VIDEO_OVERLAY_RECTANGLE (rectangle));
-  g_return_if_fail (gst_mini_object_is_writable (GST_MINI_OBJECT_CAST
-          (rectangle)));
+  g_return_if_fail (GST_MINI_OBJECT_REFCOUNT_VALUE (rectangle) == 1);
 
   rectangle->x = render_x;
   rectangle->y = render_y;
@@ -944,17 +805,12 @@ static void
 gst_video_overlay_rectangle_premultiply_0 (GstVideoFrame * frame)
 {
   int i, j;
-  int width = GST_VIDEO_FRAME_WIDTH (frame);
-  int height = GST_VIDEO_FRAME_HEIGHT (frame);
-  int stride = GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0);
-  guint8 *data = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
-
-  for (j = 0; j < height; ++j) {
+  for (j = 0; j < GST_VIDEO_FRAME_HEIGHT (frame); ++j) {
     guint8 *line;
 
-    line = data;
-    line += stride * j;
-    for (i = 0; i < width; ++i) {
+    line = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
+    line += GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0) * j;
+    for (i = 0; i < GST_VIDEO_FRAME_WIDTH (frame); ++i) {
       int a = line[0];
       line[1] = line[1] * a / 255;
       line[2] = line[2] * a / 255;
@@ -968,17 +824,12 @@ static void
 gst_video_overlay_rectangle_premultiply_3 (GstVideoFrame * frame)
 {
   int i, j;
-  int width = GST_VIDEO_FRAME_WIDTH (frame);
-  int height = GST_VIDEO_FRAME_HEIGHT (frame);
-  int stride = GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0);
-  guint8 *data = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
-
-  for (j = 0; j < height; ++j) {
+  for (j = 0; j < GST_VIDEO_FRAME_HEIGHT (frame); ++j) {
     guint8 *line;
 
-    line = data;
-    line += stride * j;
-    for (i = 0; i < width; ++i) {
+    line = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
+    line += GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0) * j;
+    for (i = 0; i < GST_VIDEO_FRAME_WIDTH (frame); ++i) {
       int a = line[3];
       line[0] = line[0] * a / 255;
       line[1] = line[1] * a / 255;
@@ -1012,17 +863,12 @@ static void
 gst_video_overlay_rectangle_unpremultiply_0 (GstVideoFrame * frame)
 {
   int i, j;
-  int width = GST_VIDEO_FRAME_WIDTH (frame);
-  int height = GST_VIDEO_FRAME_HEIGHT (frame);
-  int stride = GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0);
-  guint8 *data = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
-
-  for (j = 0; j < height; ++j) {
+  for (j = 0; j < GST_VIDEO_FRAME_HEIGHT (frame); ++j) {
     guint8 *line;
 
-    line = data;
-    line += stride * j;
-    for (i = 0; i < width; ++i) {
+    line = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
+    line += GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0) * j;
+    for (i = 0; i < GST_VIDEO_FRAME_WIDTH (frame); ++i) {
       int a = line[0];
       if (a) {
         line[1] = MIN ((line[1] * 255 + a / 2) / a, 255);
@@ -1038,17 +884,12 @@ static void
 gst_video_overlay_rectangle_unpremultiply_3 (GstVideoFrame * frame)
 {
   int i, j;
-  int width = GST_VIDEO_FRAME_WIDTH (frame);
-  int height = GST_VIDEO_FRAME_HEIGHT (frame);
-  int stride = GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0);
-  guint8 *data = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
-
-  for (j = 0; j < height; ++j) {
+  for (j = 0; j < GST_VIDEO_FRAME_HEIGHT (frame); ++j) {
     guint8 *line;
 
-    line = data;
-    line += stride * j;
-    for (i = 0; i < width; ++i) {
+    line = GST_VIDEO_FRAME_PLANE_DATA (frame, 0);
+    line += GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0) * j;
+    for (i = 0; i < GST_VIDEO_FRAME_WIDTH (frame); ++i) {
       int a = line[3];
       if (a) {
         line[0] = MIN ((line[0] * 255 + a / 2) / a, 255);
@@ -1120,15 +961,13 @@ gst_video_overlay_rectangle_apply_global_alpha (GstVideoOverlayRectangle * rect,
   GstVideoFrame frame;
   gint i, j, w, h, stride;
   gint argb_a, argb_r, argb_g, argb_b;
-  gint alpha_offset GST_UNUSED_CHECKS;
+  gint alpha_offset;
 
   g_assert (!(rect->applied_global_alpha != 1.0
           && rect->initial_alpha == NULL));
 
-#ifndef G_DISABLE_CHECKS
   alpha_offset = GST_VIDEO_INFO_COMP_POFFSET (&rect->info, 3);
   g_return_if_fail (alpha_offset == 0 || alpha_offset == 3);
-#endif
 
   if (global_alpha == rect->applied_global_alpha)
     return;
@@ -1137,13 +976,7 @@ gst_video_overlay_rectangle_apply_global_alpha (GstVideoOverlayRectangle * rect,
     gst_video_overlay_rectangle_extract_alpha (rect);
 
   src = rect->initial_alpha;
-  if (!gst_buffer_is_writable (rect->pixels)) {
-    gst_mini_object_remove_parent (GST_MINI_OBJECT_CAST (rect->pixels),
-        GST_MINI_OBJECT_CAST (rect));
-    rect->pixels = gst_buffer_copy (rect->pixels);
-    gst_mini_object_add_parent (GST_MINI_OBJECT_CAST (rect->pixels),
-        GST_MINI_OBJECT_CAST (rect));
-  }
+  rect->pixels = gst_buffer_make_writable (rect->pixels);
 
   gst_video_frame_map (&frame, &rect->info, rect->pixels, GST_MAP_READ);
   dst = GST_VIDEO_FRAME_PLANE_DATA (&frame, 0);
@@ -1160,7 +993,7 @@ gst_video_overlay_rectangle_apply_global_alpha (GstVideoOverlayRectangle * rect,
     for (j = 0; j < w; j++) {
       guint8 na = (guint8) (*src * global_alpha);
 
-      if (!!(rect->flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_PREMULTIPLIED_ALPHA)) {
+      if (! !(rect->flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_PREMULTIPLIED_ALPHA)) {
         dst[argb_r] =
             (guint8) ((double) (dst[argb_r] * 255) / (double) dst[argb_a]) *
             na / 255;
@@ -1183,9 +1016,8 @@ gst_video_overlay_rectangle_apply_global_alpha (GstVideoOverlayRectangle * rect,
 }
 
 static void
-gst_video_overlay_rectangle_convert (const GstVideoInfo * src,
-    GstBuffer * src_buffer, GstVideoFormat dest_format, GstVideoInfo * dest,
-    GstBuffer ** dest_buffer)
+gst_video_overlay_rectangle_convert (GstVideoInfo * src, GstBuffer * src_buffer,
+    GstVideoFormat dest_format, GstVideoInfo * dest, GstBuffer ** dest_buffer)
 {
   gint width, height, stride;
   GstVideoFrame src_frame, dest_frame;
@@ -1199,10 +1031,7 @@ gst_video_overlay_rectangle_convert (const GstVideoInfo * src,
   height = GST_VIDEO_INFO_HEIGHT (src);
 
   gst_video_info_init (dest);
-  if (!gst_video_info_set_format (dest, dest_format, width, height)) {
-    g_warn_if_reached ();
-    return;
-  }
+  gst_video_info_set_format (dest, dest_format, width, height);
 
   *dest_buffer = gst_buffer_new_and_alloc (GST_VIDEO_INFO_SIZE (dest));
 
@@ -1308,11 +1137,11 @@ gst_video_overlay_rectangle_get_pixels_raw_internal (GstVideoOverlayRectangle *
   format = GST_VIDEO_INFO_FORMAT (&rectangle->info);
 
   apply_global_alpha =
-      (!!(rectangle->flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_GLOBAL_ALPHA)
+      (! !(rectangle->flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_GLOBAL_ALPHA)
       && !(flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_GLOBAL_ALPHA));
   revert_global_alpha =
-      (!!(rectangle->flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_GLOBAL_ALPHA)
-      && !!(flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_GLOBAL_ALPHA));
+      (! !(rectangle->flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_GLOBAL_ALPHA)
+      && ! !(flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_GLOBAL_ALPHA));
 
   /* This assumes we don't need to adjust the format */
   if (wanted_width == width &&
@@ -1672,8 +1501,6 @@ gst_video_overlay_rectangle_set_global_alpha (GstVideoOverlayRectangle *
     rectangle, gfloat global_alpha)
 {
   g_return_if_fail (GST_IS_VIDEO_OVERLAY_RECTANGLE (rectangle));
-  g_return_if_fail (gst_mini_object_is_writable (GST_MINI_OBJECT_CAST
-          (rectangle)));
   g_return_if_fail (global_alpha >= 0 && global_alpha <= 1);
 
   if (rectangle->global_alpha != global_alpha) {
@@ -1734,7 +1561,7 @@ gst_video_overlay_rectangle_copy (GstVideoOverlayRectangle * rectangle)
  * the per rectangle sequence number, which is misleading for renderers/
  * consumers, that handle global-alpha themselves. For them  the
  * pixel-data returned by gst_video_overlay_rectangle_get_pixels_*()
- * won't be different for different global-alpha values. In this case a
+ * wont be different for different global-alpha values. In this case a
  * renderer could also use the GstBuffer pointers as a hint for changed
  * pixel-data.
  *
