@@ -27,7 +27,6 @@
 
 /**
  * SECTION:gstvideoscaler
- * @title: GstVideoScaler
  * @short_description: Utility object for rescaling video frames
  *
  * #GstVideoScaler is a utility object for rescaling and resampling
@@ -44,29 +43,6 @@
 #include "video-orc.h"
 #include "video-scaler.h"
 
-#ifndef GST_DISABLE_GST_DEBUG
-#define GST_CAT_DEFAULT ensure_debug_category()
-static GstDebugCategory *
-ensure_debug_category (void)
-{
-  static gsize cat_gonce = 0;
-
-  if (g_once_init_enter (&cat_gonce)) {
-    gsize cat_done;
-
-    cat_done = (gsize) _gst_debug_category_new ("video-scaler", 0,
-        "video-scaler object");
-
-    g_once_init_leave (&cat_gonce, cat_done);
-  }
-
-  return (GstDebugCategory *) cat_gonce;
-}
-
-#else
-#define ensure_debug_category() /* NOOP */
-#endif /* GST_DISABLE_GST_DEBUG */
-
 #define SCALE_U8          12
 #define SCALE_U8_ROUND    (1 << (SCALE_U8 -1))
 #define SCALE_U8_LQ       6
@@ -148,13 +124,9 @@ resampler_zip (GstVideoResampler * resampler, const GstVideoResampler * r1,
 static void
 realloc_tmplines (GstVideoScaler * scale, gint n_elems, gint width)
 {
-  gint n_taps = scale->resampler.max_taps;
-
-  if (scale->flags & GST_VIDEO_SCALER_FLAG_INTERLACED)
-    n_taps *= 2;
-
   scale->tmpline1 =
-      g_realloc (scale->tmpline1, sizeof (gint32) * width * n_elems * n_taps);
+      g_realloc (scale->tmpline1,
+      sizeof (gint32) * width * n_elems * scale->resampler.max_taps);
   scale->tmpline2 =
       g_realloc (scale->tmpline2, sizeof (gint32) * width * n_elems);
   scale->tmpwidth = width;
@@ -190,10 +162,8 @@ scaler_dump (GstVideoScaler * scale)
 #endif
 }
 
-#define INTERLACE_SHIFT 0.5
-
 /**
- * gst_video_scaler_new: (skip)
+ * gst_video_scaler_new:
  * @method: a #GstVideoResamplerMethod
  * @flags: #GstVideoScalerFlags
  * @n_taps: number of taps to use
@@ -208,7 +178,7 @@ scaler_dump (GstVideoScaler * scale)
  * pixel. If n_taps is 0, this function chooses a good value automatically based
  * on the @method and @in_size/@out_size.
  *
- * Returns: a #GstVideoScaler
+ * Returns: a #GstVideoResample
  */
 GstVideoScaler *
 gst_video_scaler_new (GstVideoResamplerMethod method, GstVideoScalerFlags flags,
@@ -219,7 +189,7 @@ gst_video_scaler_new (GstVideoResamplerMethod method, GstVideoScalerFlags flags,
   g_return_val_if_fail (in_size != 0, NULL);
   g_return_val_if_fail (out_size != 0, NULL);
 
-  scale = g_new0 (GstVideoScaler, 1);
+  scale = g_slice_new0 (GstVideoScaler);
 
   GST_DEBUG ("%d %u  %u->%u", method, n_taps, in_size, out_size);
 
@@ -228,18 +198,14 @@ gst_video_scaler_new (GstVideoResamplerMethod method, GstVideoScalerFlags flags,
 
   if (flags & GST_VIDEO_SCALER_FLAG_INTERLACED) {
     GstVideoResampler tresamp, bresamp;
-    gdouble shift;
 
-    shift = (INTERLACE_SHIFT * out_size) / in_size;
-
-    gst_video_resampler_init (&tresamp, method,
-        GST_VIDEO_RESAMPLER_FLAG_HALF_TAPS, (out_size + 1) / 2, n_taps, shift,
-        (in_size + 1) / 2, (out_size + 1) / 2, options);
+    gst_video_resampler_init (&tresamp, method, 0, (out_size + 1) / 2, n_taps,
+        -0.5, (in_size + 1) / 2, (out_size + 1) / 2, options);
 
     n_taps = tresamp.max_taps;
 
     gst_video_resampler_init (&bresamp, method, 0, out_size - tresamp.out_size,
-        n_taps, -shift, in_size - tresamp.in_size,
+        n_taps, 0.5, in_size - tresamp.in_size,
         out_size - tresamp.out_size, options);
 
     resampler_zip (&scale->resampler, &tresamp, &bresamp);
@@ -279,31 +245,15 @@ gst_video_scaler_free (GstVideoScaler * scale)
   g_free (scale->offset_n);
   g_free (scale->tmpline1);
   g_free (scale->tmpline2);
-  g_free (scale);
-}
-
-/**
- * gst_video_scaler_get_max_taps:
- * @scale: a #GstVideoScaler
- *
- * Get the maximum number of taps for @scale.
- *
- * Returns: the maximum number of taps
- */
-guint
-gst_video_scaler_get_max_taps (GstVideoScaler * scale)
-{
-  g_return_val_if_fail (scale != NULL, 0);
-
-  return scale->resampler.max_taps;
+  g_slice_free (GstVideoScaler, scale);
 }
 
 /**
  * gst_video_scaler_get_coeff:
  * @scale: a #GstVideoScaler
  * @out_offset: an output offset
- * @in_offset: (out) (optional): result input offset
- * @n_taps: (out) (optional): result n_taps
+ * @in_offset: result input offset
+ * @n_taps: result n_taps
  *
  * For a given pixel at @out_offset, get the first required input pixel at
  * @in_offset and the @n_taps filter coefficients.
@@ -382,7 +332,7 @@ resampler_convert_coeff (const gdouble * src,
   }
 
   if (!exact)
-    GST_DEBUG ("can't find exact taps");
+    GST_WARNING ("can't find exact taps");
 
   return exact;
 }
@@ -1119,7 +1069,7 @@ get_y_offset (GstVideoFormat format)
 }
 
 /**
- * gst_video_scaler_combine_packed_YUV: (skip)
+ * gst_video_scaler_combine_packed_YUV:
  * @y_scale: a scaler for the Y component
  * @uv_scale: a scaler for the U and V components
  * @in_format: the input video format
@@ -1144,10 +1094,8 @@ gst_video_scaler_combine_packed_YUV (GstVideoScaler * y_scale,
 
   g_return_val_if_fail (y_scale != NULL, NULL);
   g_return_val_if_fail (uv_scale != NULL, NULL);
-  g_return_val_if_fail (uv_scale->resampler.max_taps ==
-      y_scale->resampler.max_taps, NULL);
 
-  scale = g_new0 (GstVideoScaler, 1);
+  scale = g_slice_new0 (GstVideoScaler);
 
   scale->method = y_scale->method;
   scale->flags = y_scale->flags;
@@ -1155,8 +1103,8 @@ gst_video_scaler_combine_packed_YUV (GstVideoScaler * y_scale,
 
   resampler = &scale->resampler;
 
-  out_size = GST_ROUND_UP_4 (y_scale->resampler.out_size * 2);
-  max_taps = y_scale->resampler.max_taps;
+  out_size = y_scale->resampler.out_size * 2;
+  max_taps = MAX (y_scale->resampler.max_taps, uv_scale->resampler.max_taps);
   n_phases = out_size;
   offset = g_malloc (sizeof (guint32) * out_size);
   phase = g_malloc (sizeof (guint32) * n_phases);
@@ -1173,21 +1121,17 @@ gst_video_scaler_combine_packed_YUV (GstVideoScaler * y_scale,
 
   scale->in_y_offset = get_y_offset (in_format);
   scale->out_y_offset = get_y_offset (out_format);
-  scale->inc = y_scale->inc;
 
   for (i = 0; i < out_size; i++) {
-    gint ic;
-
     if ((i & 1) == scale->out_y_offset) {
-      ic = MIN (i / 2, y_scale->resampler.out_size - 1);
-      offset[i] = y_scale->resampler.offset[ic] * 2 + scale->in_y_offset;
+      offset[i] = y_scale->resampler.offset[i / 2] * 2 + scale->in_y_offset;
       memcpy (taps + i * max_taps, y_scale->resampler.taps +
-          y_scale->resampler.phase[ic] * max_taps, max_taps * sizeof (gdouble));
+          y_scale->resampler.phase[i / 2] * max_taps,
+          max_taps * sizeof (gdouble));
     } else {
-      ic = MIN (i / 4, uv_scale->resampler.out_size - 1);
-      offset[i] = uv_scale->resampler.offset[ic] * 4 + (i & 3);
+      offset[i] = uv_scale->resampler.offset[i / 4] * 4 + (i & 3);
       memcpy (taps + i * max_taps, uv_scale->resampler.taps +
-          uv_scale->resampler.phase[ic] * max_taps,
+          uv_scale->resampler.phase[i / 4] * max_taps,
           max_taps * sizeof (gdouble));
     }
     phase[i] = i;
@@ -1202,28 +1146,28 @@ static gboolean
 get_functions (GstVideoScaler * hscale, GstVideoScaler * vscale,
     GstVideoFormat format,
     GstVideoScalerHFunc * hfunc, GstVideoScalerVFunc * vfunc,
-    gint * n_elems, guint * width, gint * bits)
+    gint * n_elems, gint * width_mult)
 {
-  gboolean mono = FALSE;
+  gint bits;
+
+  *width_mult = 1;
 
   switch (format) {
     case GST_VIDEO_FORMAT_GRAY8:
-      *bits = 8;
+      bits = 8;
       *n_elems = 1;
-      mono = TRUE;
       break;
     case GST_VIDEO_FORMAT_YUY2:
     case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
-      *bits = 8;
+      bits = 8;
       *n_elems = 1;
-      *width = GST_ROUND_UP_4 (*width * 2);
+      *width_mult = 2;
       break;
     case GST_VIDEO_FORMAT_RGB:
     case GST_VIDEO_FORMAT_BGR:
     case GST_VIDEO_FORMAT_v308:
-    case GST_VIDEO_FORMAT_IYU2:
-      *bits = 8;
+      bits = 8;
       *n_elems = 3;
       break;
     case GST_VIDEO_FORMAT_AYUV:
@@ -1235,40 +1179,31 @@ get_functions (GstVideoScaler * hscale, GstVideoScaler * vscale,
     case GST_VIDEO_FORMAT_BGRA:
     case GST_VIDEO_FORMAT_ARGB:
     case GST_VIDEO_FORMAT_ABGR:
-      *bits = 8;
+      bits = 8;
       *n_elems = 4;
       break;
     case GST_VIDEO_FORMAT_ARGB64:
-    case GST_VIDEO_FORMAT_ARGB64_LE:
-    case GST_VIDEO_FORMAT_ARGB64_BE:
-    case GST_VIDEO_FORMAT_RGBA64_BE:
-    case GST_VIDEO_FORMAT_RGBA64_LE:
-    case GST_VIDEO_FORMAT_BGRA64_BE:
-    case GST_VIDEO_FORMAT_BGRA64_LE:
-    case GST_VIDEO_FORMAT_ABGR64_BE:
-    case GST_VIDEO_FORMAT_ABGR64_LE:
     case GST_VIDEO_FORMAT_AYUV64:
-      *bits = 16;
+      bits = 16;
       *n_elems = 4;
       break;
     case GST_VIDEO_FORMAT_GRAY16_LE:
     case GST_VIDEO_FORMAT_GRAY16_BE:
-      *bits = 16;
+      bits = 16;
       *n_elems = 1;
-      mono = TRUE;
       break;
     case GST_VIDEO_FORMAT_NV12:
     case GST_VIDEO_FORMAT_NV16:
     case GST_VIDEO_FORMAT_NV21:
     case GST_VIDEO_FORMAT_NV24:
     case GST_VIDEO_FORMAT_NV61:
-      *bits = 8;
+      bits = 8;
       *n_elems = 2;
       break;
     default:
       return FALSE;
   }
-  if (*bits == 8) {
+  if (bits == 8) {
     switch (hscale ? hscale->resampler.max_taps : 0) {
       case 0:
         break;
@@ -1283,7 +1218,7 @@ get_functions (GstVideoScaler * hscale, GstVideoScaler * vscale,
           *hfunc = video_scale_h_near_u32;
         break;
       case 2:
-        if (*n_elems == 1 && mono)
+        if (*n_elems == 1 && *width_mult == 1)
           *hfunc = video_scale_h_2tap_1u8;
         else if (*n_elems == 4)
           *hfunc = video_scale_h_2tap_4u8;
@@ -1310,7 +1245,7 @@ get_functions (GstVideoScaler * hscale, GstVideoScaler * vscale,
         *vfunc = video_scale_v_ntap_u8;
         break;
     }
-  } else if (*bits == 16) {
+  } else if (bits == 16) {
     switch (hscale ? hscale->resampler.max_taps : 0) {
       case 0:
         break;
@@ -1357,18 +1292,19 @@ void
 gst_video_scaler_horizontal (GstVideoScaler * scale, GstVideoFormat format,
     gpointer src, gpointer dest, guint dest_offset, guint width)
 {
-  gint n_elems, bits;
-  GstVideoScalerHFunc func = NULL;
+  gint n_elems, mult;
+  GstVideoScalerHFunc func;
 
   g_return_if_fail (scale != NULL);
   g_return_if_fail (src != NULL);
   g_return_if_fail (dest != NULL);
   g_return_if_fail (dest_offset + width <= scale->resampler.out_size);
 
-  if (!get_functions (scale, NULL, format, &func, NULL, &n_elems, &width, &bits)
-      || func == NULL)
+  if (!get_functions (scale, NULL, format, &func, NULL, &n_elems, &mult))
     goto no_func;
 
+  width *= mult;
+
   if (scale->tmpwidth < width)
     realloc_tmplines (scale, n_elems, width);
 
@@ -1392,29 +1328,30 @@ no_func:
  *
  * Vertically combine @width pixels in the lines in @src_lines to @dest.
  * @dest is the location of the target line at @dest_offset and
- * @srcs are the input lines for @dest_offset.
+ * @srcs are the input lines for @dest_offset, as obtained with
+ * gst_video_scaler_get_info().
  */
 void
 gst_video_scaler_vertical (GstVideoScaler * scale, GstVideoFormat format,
     gpointer src_lines[], gpointer dest, guint dest_offset, guint width)
 {
-  gint n_elems, bits;
-  GstVideoScalerVFunc func = NULL;
+  gint n_elems, mult;
+  GstVideoScalerVFunc func;
 
   g_return_if_fail (scale != NULL);
   g_return_if_fail (src_lines != NULL);
   g_return_if_fail (dest != NULL);
   g_return_if_fail (dest_offset < scale->resampler.out_size);
 
-  if (!get_functions (NULL, scale, format, NULL, &func, &n_elems, &width, &bits)
-      || func == NULL)
+  if (!get_functions (NULL, scale, format, NULL, &func, &n_elems, &mult))
     goto no_func;
 
+  width *= mult;
+
   if (scale->tmpwidth < width)
     realloc_tmplines (scale, n_elems, width);
 
   func (scale, src_lines, dest, dest_offset, width, n_elems);
-
   return;
 
 no_func:
@@ -1426,7 +1363,7 @@ no_func:
 
 /**
  * gst_video_scaler_2d:
- * @hscale: a horizontal #GstVideoScaler
+ * @hscale: a horzontal #GstVideoScaler
  * @vscale: a vertical #GstVideoScaler
  * @format: a #GstVideoFormat for @srcs and @dest
  * @src: source pixels
@@ -1453,23 +1390,21 @@ gst_video_scaler_2d (GstVideoScaler * hscale, GstVideoScaler * vscale,
     gpointer dest, gint dest_stride, guint x, guint y,
     guint width, guint height)
 {
-  gint n_elems, bits;
-  GstVideoScalerHFunc hfunc = NULL;
-  GstVideoScalerVFunc vfunc = NULL;
+  gint n_elems, mult;
+  GstVideoScalerHFunc hfunc;
+  GstVideoScalerVFunc vfunc;
   gint i;
-  gboolean interlaced;
 
   g_return_if_fail (src != NULL);
   g_return_if_fail (dest != NULL);
 
-  if (!get_functions (hscale, vscale, format, &hfunc, &vfunc, &n_elems, &width,
-          &bits))
+  if (!get_functions (hscale, vscale, format, &hfunc, &vfunc, &n_elems, &mult))
     goto no_func;
 
-  interlaced = vscale && !!(vscale->flags & GST_VIDEO_SCALER_FLAG_INTERLACED);
+  width *= mult;
 
 #define LINE(s,ss,i)  ((guint8 *)(s) + ((i) * (ss)))
-#define TMP_LINE(s,i) ((guint8 *)((s)->tmpline1) + (i) * (sizeof (gint32) * width * n_elems))
+#define TMP_LINE(s,i,v) ((guint8 *)(s->tmpline1) + (((i) % (v)) * (sizeof (gint32) * width * n_elems)))
 
   if (vscale == NULL) {
     if (hscale == NULL) {
@@ -1477,7 +1412,7 @@ gst_video_scaler_2d (GstVideoScaler * hscale, GstVideoScaler * vscale,
       guint8 *s, *d;
 
       xo = x * n_elems;
-      xw = width * n_elems * (bits / 8);
+      xw = width * n_elems;
 
       s = LINE (src, src_stride, y) + xo;
       d = LINE (dest, dest_stride, y) + xo;
@@ -1506,35 +1441,22 @@ gst_video_scaler_2d (GstVideoScaler * hscale, GstVideoScaler * vscale,
       realloc_tmplines (vscale, n_elems, width);
 
     v_taps = vscale->resampler.max_taps;
-
-    lines = g_alloca ((interlaced ? 2 : 1) * v_taps * sizeof (gpointer));
-    memset (lines, 0, (interlaced ? 2 : 1) * v_taps * sizeof (gpointer));
+    lines = g_alloca (v_taps * sizeof (gpointer));
 
     if (hscale == NULL) {
-      guint src_inc = interlaced ? 2 : 1;
-
       /* only vertical scaling */
       for (i = y; i < height; i++) {
         guint in, j;
 
         in = vscale->resampler.offset[i];
-        for (j = 0; j < v_taps; j++) {
-          guint l = in + j * src_inc;
-
-          g_assert (l < vscale->resampler.in_size);
-          lines[j * src_inc] = LINE (src, src_stride, l);
-        }
+        for (j = 0; j < v_taps; j++)
+          lines[j] = LINE (src, src_stride, in + j);
 
         vfunc (vscale, lines, LINE (dest, dest_stride, i), i, width, n_elems);
       }
     } else {
+      gint tmp_in = y;
       gint s1, s2;
-      guint *tmpline_lines;
-
-      tmpline_lines = g_newa (guint, (interlaced ? 2 : 1) * v_taps);
-      /* initialize with -1 */
-      memset (tmpline_lines, 0xff,
-          (interlaced ? 2 : 1) * v_taps * sizeof (guint));
 
       if (hscale->tmpwidth < width)
         realloc_tmplines (hscale, n_elems, width);
@@ -1545,91 +1467,54 @@ gst_video_scaler_2d (GstVideoScaler * hscale, GstVideoScaler * vscale,
       if (s1 <= s2) {
         for (i = y; i < height; i++) {
           guint in, j;
-          guint src_inc = interlaced ? 2 : 1;
-          guint f2_offset = (interlaced && (i % 2 == 1)) * v_taps;
 
           in = vscale->resampler.offset[i];
-          for (j = 0; j < v_taps; j++) {
-            guint k;
-            guint l = in + j * src_inc;
-
-            g_assert (l < vscale->resampler.in_size);
-
-            /* First check if we already have this line in tmplines */
-            for (k = f2_offset; k < v_taps + f2_offset; k++) {
-              if (tmpline_lines[k] == l) {
-                lines[j * src_inc] = TMP_LINE (vscale, k);
-                break;
-              }
-            }
-            /* Found */
-            if (k < v_taps + f2_offset)
-              continue;
-
-            /* Otherwise find an empty line we can clear */
-            for (k = f2_offset; k < v_taps + f2_offset; k++) {
-              if (tmpline_lines[k] < in || tmpline_lines[k] == -1)
-                break;
-            }
-
-            /* Must not happen, that would mean we don't have enough space to
-             * begin with */
-            g_assert (k < v_taps + f2_offset);
-
-            hfunc (hscale, LINE (src, src_stride, l), TMP_LINE (vscale, k), x,
-                width, n_elems);
-            tmpline_lines[k] = l;
-            lines[j * src_inc] = TMP_LINE (vscale, k);
+          while (tmp_in < in)
+            tmp_in++;
+          while (tmp_in < in + v_taps) {
+            hfunc (hscale, LINE (src, src_stride, tmp_in), TMP_LINE (vscale,
+                    tmp_in, v_taps), x, width, n_elems);
+            tmp_in++;
           }
+          for (j = 0; j < v_taps; j++)
+            lines[j] = TMP_LINE (vscale, in + j, v_taps);
 
           vfunc (vscale, lines, LINE (dest, dest_stride, i), i, width, n_elems);
         }
       } else {
-        guint vx, vw, w1, ws;
+        guint vx, vw, w1;
         guint h_taps;
 
         h_taps = hscale->resampler.max_taps;
         w1 = x + width - 1;
-        ws = hscale->resampler.offset[w1];
 
-        /* we need to estimate the area that we first need to scale in the
-         * vertical direction. Scale x and width to find the lower bound and
-         * overshoot the width to find the upper bound */
         vx = (hscale->inc * x) >> 16;
         vx = MIN (vx, hscale->resampler.offset[x]);
         vw = (hscale->inc * (x + width)) >> 16;
         if (hscale->merged) {
           if ((w1 & 1) == hscale->out_y_offset)
-            vw = MAX (vw, ws + (2 * h_taps));
+            vw = MAX (vw, hscale->resampler.offset[w1] + (2 * h_taps));
           else
-            vw = MAX (vw, ws + (4 * h_taps));
+            vw = MAX (vw, hscale->resampler.offset[w1] + (4 * h_taps));
         } else {
-          vw = MAX (vw, ws + h_taps);
+          vw = MAX (vw, hscale->resampler.offset[w1] + h_taps);
         }
         vw += 1;
-        /* but clamp to max size */
-        vw = MIN (vw, hscale->resampler.in_size);
 
         if (vscale->tmpwidth < vw)
           realloc_tmplines (vscale, n_elems, vw);
 
         for (i = y; i < height; i++) {
           guint in, j;
-          guint src_inc = interlaced ? 2 : 1;
 
           in = vscale->resampler.offset[i];
-          for (j = 0; j < v_taps; j++) {
-            guint l = in + j * src_inc;
+          for (j = 0; j < v_taps; j++)
+            lines[j] = LINE (src, src_stride, in + j) + vx * n_elems;
 
-            g_assert (l < vscale->resampler.in_size);
-            lines[j * src_inc] =
-                LINE (src, src_stride, in + j * src_inc) + vx * n_elems;
-          }
-
-          vfunc (vscale, lines, TMP_LINE (vscale, 0) + vx * n_elems, i,
+          vfunc (vscale, lines, TMP_LINE (vscale, 0, v_taps) + vx * n_elems, i,
               vw - vx, n_elems);
 
-          hfunc (hscale, TMP_LINE (vscale, 0), LINE (dest, dest_stride,
+          hfunc (hscale, TMP_LINE (vscale, 0, v_taps), LINE (dest, dest_stride,
                   i), x, width, n_elems);
         }
       }
@@ -1642,6 +1527,3 @@ no_func:
     GST_WARNING ("no scaler function for format");
   }
 }
-
-#undef LINE
-#undef TMP_LINE
