@@ -31,18 +31,20 @@
 
 /**
  * SECTION:gstvideo
- * @title: GstVideoAlignment
  * @short_description: Support library for video operations
  *
+ * <refsect2>
+ * <para>
  * This library contains some helper functions and includes the
  * videosink and videofilter base classes.
- *
+ * </para>
+ * </refsect2>
  */
 
 /**
  * gst_video_calculate_display_ratio:
- * @dar_n: (out): Numerator of the calculated display_ratio
- * @dar_d: (out): Denominator of the calculated display_ratio
+ * @dar_n: Numerator of the calculated display_ratio
+ * @dar_d: Denominator of the calculated display_ratio
  * @video_width: Width of the video frame in pixels
  * @video_height: Height of the video frame in pixels
  * @video_par_n: Numerator of the pixel aspect ratio of the input video.
@@ -131,8 +133,8 @@ gst_video_guess_framerate (GstClockTime duration, gint * dest_n, gint * dest_d)
   /* Use a limited precision conversion by default for more sensible results,
    * unless the frame duration is absurdly small (high speed cameras?) */
   if (duration > 100000) {
-    best_n = GST_SECOND / 10000;
-    best_d = duration / 10000;
+    best_n = 10000;
+    best_d = duration / 100000;
   } else {
     best_n = GST_SECOND;
     best_d = duration;
