@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -25,12 +25,21 @@
  * audioresample resamples raw audio buffers to different sample rates using
  * a configurable windowing function to enhance quality.
  *
+ * By default, the resampler uses a reduced sinc table, with cubic interpolation filling in
+ * the gaps. This ensures that the table does not become too big. However, the interpolation
+ * increases the CPU usage considerably. As an alternative, a full sinc table can be used.
+ * Doing so can drastically reduce CPU usage (4x faster with 44.1 -> 48 kHz conversions for
+ * example), at the cost of increased memory consumption, plus the sinc table takes longer
+ * to initialize when the element is created. A third mode exists, which uses the full table
+ * unless said table would become too large, in which case the interpolated one is used instead.
+ *
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! audio/x-raw, rate=8000 ! alsasink
- * ]| Decode an Ogg/Vorbis downsample to 8Khz and play sound through alsa.
+ * gst-launch-1.0 -v uridecodebin uri=file:///path/to/audio.ogg ! audioconvert ! audioresample ! audio/x-raw, rate=8000 ! autoaudiosink
+ * ]| Decode an audio file and downsample it to 8Khz and play sound.
  * To create the Ogg/Vorbis file refer to the documentation of vorbisenc.
+ * This assumes there is an audio sink that will accept/handle 8kHz audio.
  * </refsect2>
  */
 
@@ -62,10 +71,14 @@ GST_DEBUG_CATEGORY (audio_resample_debug);
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_PERFORMANCE);
 #endif
 
+#define GST_TYPE_SPEEX_RESAMPLER_SINC_FILTER_MODE (speex_resampler_sinc_filter_mode_get_type ())
+
 enum
 {
   PROP_0,
-  PROP_QUALITY
+  PROP_QUALITY,
+  PROP_SINC_FILTER_MODE,
+  PROP_SINC_FILTER_AUTO_THRESHOLD
 };
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
@@ -104,6 +117,8 @@ static void gst_audio_resample_set_property (GObject * object,
 static void gst_audio_resample_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec);
 
+static GType speex_resampler_sinc_filter_mode_get_type (void);
+
 /* vmethods */
 static gboolean gst_audio_resample_get_unit_size (GstBaseTransform * base,
     GstCaps * caps, gsize * size);
@@ -118,6 +133,8 @@ static gboolean gst_audio_resample_set_caps (GstBaseTransform * base,
     GstCaps * incaps, GstCaps * outcaps);
 static GstFlowReturn gst_audio_resample_transform (GstBaseTransform * base,
     GstBuffer * inbuf, GstBuffer * outbuf);
+static gboolean gst_audio_resample_transform_meta (GstBaseTransform * trans,
+    GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf);
 static gboolean gst_audio_resample_sink_event (GstBaseTransform * base,
     GstEvent * event);
 static gboolean gst_audio_resample_start (GstBaseTransform * base);
@@ -144,6 +161,21 @@ gst_audio_resample_class_init (GstAudioResampleClass * klass)
           SPEEX_RESAMPLER_QUALITY_DEFAULT,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_SINC_FILTER_MODE,
+      g_param_spec_enum ("sinc-filter-mode", "Sinc filter table mode",
+          "What sinc filter table mode to use",
+          GST_TYPE_SPEEX_RESAMPLER_SINC_FILTER_MODE,
+          SPEEX_RESAMPLER_SINC_FILTER_DEFAULT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class,
+      PROP_SINC_FILTER_AUTO_THRESHOLD,
+      g_param_spec_uint ("sinc-filter-auto-threshold",
+          "Sinc filter auto mode threshold",
+          "Memory usage threshold to use if sinc filter mode is AUTO, given in bytes",
+          0, G_MAXUINT, SPEEX_RESAMPLER_SINC_FILTER_AUTO_THRESHOLD_DEFAULT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&gst_audio_resample_src_template));
   gst_element_class_add_pad_template (gstelement_class,
@@ -171,6 +203,8 @@ gst_audio_resample_class_init (GstAudioResampleClass * klass)
       GST_DEBUG_FUNCPTR (gst_audio_resample_transform);
   GST_BASE_TRANSFORM_CLASS (klass)->sink_event =
       GST_DEBUG_FUNCPTR (gst_audio_resample_sink_event);
+  GST_BASE_TRANSFORM_CLASS (klass)->transform_meta =
+      GST_DEBUG_FUNCPTR (gst_audio_resample_transform_meta);
 
   GST_BASE_TRANSFORM_CLASS (klass)->passthrough_on_same_caps = TRUE;
 }
@@ -181,6 +215,9 @@ gst_audio_resample_init (GstAudioResample * resample)
   GstBaseTransform *trans = GST_BASE_TRANSFORM (resample);
 
   resample->quality = SPEEX_RESAMPLER_QUALITY_DEFAULT;
+  resample->sinc_filter_mode = SPEEX_RESAMPLER_SINC_FILTER_DEFAULT;
+  resample->sinc_filter_auto_threshold =
+      SPEEX_RESAMPLER_SINC_FILTER_AUTO_THRESHOLD_DEFAULT;
 
   gst_base_transform_set_gap_aware (trans, TRUE);
   gst_pad_set_query_function (trans->srcpad, gst_audio_resample_query);
@@ -349,13 +386,16 @@ gst_audio_resample_get_funcs (gint width, gboolean fp)
 
 static SpeexResamplerState *
 gst_audio_resample_init_state (GstAudioResample * resample, gint width,
-    gint channels, gint inrate, gint outrate, gint quality, gboolean fp)
+    gint channels, gint inrate, gint outrate, gint quality, gboolean fp,
+    SpeexResamplerSincFilterMode sinc_filter_mode,
+    guint32 sinc_filter_auto_threshold)
 {
   SpeexResamplerState *ret = NULL;
   gint err = RESAMPLER_ERR_SUCCESS;
   const SpeexResampleFuncs *funcs = gst_audio_resample_get_funcs (width, fp);
 
-  ret = funcs->init (channels, inrate, outrate, quality, &err);
+  ret = funcs->init (channels, inrate, outrate, quality,
+      sinc_filter_mode, sinc_filter_auto_threshold, &err);
 
   if (G_UNLIKELY (err != RESAMPLER_ERR_SUCCESS)) {
     GST_ERROR_OBJECT (resample, "Failed to create resampler state: %s",
@@ -363,6 +403,11 @@ gst_audio_resample_init_state (GstAudioResample * resample, gint width,
     return NULL;
   }
 
+  if (sinc_filter_mode == SPEEX_RESAMPLER_SINC_FILTER_AUTO) {
+    GST_INFO_OBJECT (resample, "Using the %s sinc filter table",
+        funcs->get_sinc_filter_mode (ret) ? "full" : "interpolated");
+  }
+
   funcs->skip_zeros (ret);
 
   return ret;
@@ -370,7 +415,9 @@ gst_audio_resample_init_state (GstAudioResample * resample, gint width,
 
 static gboolean
 gst_audio_resample_update_state (GstAudioResample * resample, gint width,
-    gint channels, gint inrate, gint outrate, gint quality, gboolean fp)
+    gint channels, gint inrate, gint outrate, gint quality, gboolean fp,
+    SpeexResamplerSincFilterMode sinc_filter_mode,
+    guint32 sinc_filter_auto_threshold)
 {
   gboolean ret = TRUE;
   gboolean updated_latency = FALSE;
@@ -381,11 +428,13 @@ gst_audio_resample_update_state (GstAudioResample * resample, gint width,
   if (resample->state == NULL) {
     ret = TRUE;
   } else if (resample->channels != channels || fp != resample->fp
-      || width != resample->width) {
+      || width != resample->width
+      || sinc_filter_mode != resample->sinc_filter_mode
+      || sinc_filter_auto_threshold != resample->sinc_filter_auto_threshold) {
     resample->funcs->destroy (resample->state);
     resample->state =
         gst_audio_resample_init_state (resample, width, channels, inrate,
-        outrate, quality, fp);
+        outrate, quality, fp, sinc_filter_mode, sinc_filter_auto_threshold);
 
     resample->funcs = gst_audio_resample_get_funcs (width, fp);
     ret = (resample->state != NULL);
@@ -417,6 +466,8 @@ gst_audio_resample_update_state (GstAudioResample * resample, gint width,
   resample->quality = quality;
   resample->inrate = inrate;
   resample->outrate = outrate;
+  resample->sinc_filter_mode = sinc_filter_mode;
+  resample->sinc_filter_auto_threshold = sinc_filter_auto_threshold;
 
   if (updated_latency)
     gst_element_post_message (GST_ELEMENT (resample),
@@ -526,7 +577,8 @@ gst_audio_resample_set_caps (GstBaseTransform * base, GstCaps * incaps,
 
   ret =
       gst_audio_resample_update_state (resample, width, channels, inrate,
-      outrate, resample->quality, fp);
+      outrate, resample->quality, fp, resample->sinc_filter_mode,
+      resample->sinc_filter_auto_threshold);
 
   if (G_UNLIKELY (!ret))
     return FALSE;
@@ -1126,7 +1178,8 @@ gst_audio_resample_transform (GstBaseTransform * base, GstBuffer * inbuf,
     if (G_UNLIKELY (!(resample->state =
                 gst_audio_resample_init_state (resample, resample->width,
                     resample->channels, resample->inrate, resample->outrate,
-                    resample->quality, resample->fp))))
+                    resample->quality, resample->fp, resample->sinc_filter_mode,
+                    resample->sinc_filter_auto_threshold))))
       return GST_FLOW_ERROR;
 
     resample->funcs =
@@ -1197,6 +1250,23 @@ gst_audio_resample_transform (GstBaseTransform * base, GstBuffer * inbuf,
   return GST_FLOW_OK;
 }
 
+static gboolean
+gst_audio_resample_transform_meta (GstBaseTransform * trans, GstBuffer * outbuf,
+    GstMeta * meta, GstBuffer * inbuf)
+{
+  const GstMetaInfo *info = meta->info;
+  const gchar *const *tags;
+
+  tags = gst_meta_api_type_get_tags (info->api);
+
+  if (tags && g_strv_length ((gchar **) tags) == 1
+      && gst_meta_api_type_has_tag (info->api,
+          g_quark_from_string (GST_META_TAG_AUDIO_STR)))
+    return TRUE;
+
+  return FALSE;
+}
+
 static gboolean
 gst_audio_resample_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
@@ -1279,8 +1349,31 @@ gst_audio_resample_set_property (GObject * object, guint prop_id,
 
       gst_audio_resample_update_state (resample, resample->width,
           resample->channels, resample->inrate, resample->outrate,
-          quality, resample->fp);
+          quality, resample->fp, resample->sinc_filter_mode,
+          resample->sinc_filter_auto_threshold);
       break;
+    case PROP_SINC_FILTER_MODE:{
+      /* FIXME locking! */
+      SpeexResamplerSincFilterMode sinc_filter_mode = g_value_get_enum (value);
+
+      gst_audio_resample_update_state (resample, resample->width,
+          resample->channels, resample->inrate, resample->outrate,
+          resample->quality, resample->fp, sinc_filter_mode,
+          resample->sinc_filter_auto_threshold);
+
+      break;
+    }
+    case PROP_SINC_FILTER_AUTO_THRESHOLD:{
+      /* FIXME locking! */
+      guint32 sinc_filter_auto_threshold = g_value_get_uint (value);
+
+      gst_audio_resample_update_state (resample, resample->width,
+          resample->channels, resample->inrate, resample->outrate,
+          resample->quality, resample->fp, resample->sinc_filter_mode,
+          sinc_filter_auto_threshold);
+
+      break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1299,12 +1392,41 @@ gst_audio_resample_get_property (GObject * object, guint prop_id,
     case PROP_QUALITY:
       g_value_set_int (value, resample->quality);
       break;
+    case PROP_SINC_FILTER_MODE:
+      g_value_set_enum (value, resample->sinc_filter_mode);
+      break;
+    case PROP_SINC_FILTER_AUTO_THRESHOLD:
+      g_value_set_uint (value, resample->sinc_filter_auto_threshold);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
+static GType
+speex_resampler_sinc_filter_mode_get_type (void)
+{
+  static GType speex_resampler_sinc_filter_mode_type = 0;
+
+  if (!speex_resampler_sinc_filter_mode_type) {
+    static const GEnumValue sinc_filter_modes[] = {
+      {SPEEX_RESAMPLER_SINC_FILTER_INTERPOLATED, "Use interpolated sinc table",
+          "interpolated"},
+      {SPEEX_RESAMPLER_SINC_FILTER_FULL, "Use full sinc table", "full"},
+      {SPEEX_RESAMPLER_SINC_FILTER_AUTO,
+          "Use full table if table size below threshold", "auto"},
+      {0, NULL, NULL},
+    };
+
+    speex_resampler_sinc_filter_mode_type =
+        g_enum_register_static ("SpeexResamplerSincFilterMode",
+        sinc_filter_modes);
+  }
+
+  return speex_resampler_sinc_filter_mode_type;
+}
+
 /* FIXME: should have a benchmark fallback for the case where orc is disabled */
 #if defined(AUDIORESAMPLE_FORMAT_AUTO) && !defined(DISABLE_ORC)
 
@@ -1367,13 +1489,17 @@ _benchmark_integer_resampling (void)
   orc_profile_init (&a);
   orc_profile_init (&b);
 
-  sta = resample_float_resampler_init (1, 48000, 24000, 4, NULL);
+  sta = resample_float_resampler_init (1, 48000, 24000, 4,
+      SPEEX_RESAMPLER_SINC_FILTER_INTERPOLATED,
+      SPEEX_RESAMPLER_SINC_FILTER_AUTO_THRESHOLD_DEFAULT, NULL);
   if (sta == NULL) {
     GST_ERROR ("Failed to create float resampler state");
     return FALSE;
   }
 
-  stb = resample_int_resampler_init (1, 48000, 24000, 4, NULL);
+  stb = resample_int_resampler_init (1, 48000, 24000, 4,
+      SPEEX_RESAMPLER_SINC_FILTER_INTERPOLATED,
+      SPEEX_RESAMPLER_SINC_FILTER_AUTO_THRESHOLD_DEFAULT, NULL);
   if (stb == NULL) {
     resample_float_resampler_destroy (sta);
     GST_ERROR ("Failed to create int resampler state");
