@@ -29,7 +29,7 @@
  * file descriptors can be added to multisocketsink by emitting the #GstMultiSocketSink::add signal. 
  * For each descriptor added, the #GstMultiSocketSink::client-added signal will be called.
  *
- * As of version 0.10.8, a client can also be added with the #GstMultiSocketSink::add-full signal
+ * A client can also be added with the #GstMultiSocketSink::add-full signal
  * that allows for more control over what and how much data a client 
  * initially receives.
  *
@@ -61,7 +61,7 @@
  * Multisocketsink will always keep at least one keyframe in its internal buffers
  * when the sync-mode is set to latest-keyframe.
  *
- * As of version 0.10.8, there are additional values for the #GstMultiSocketSink:sync-method 
+ * There are additional values for the #GstMultiSocketSink:sync-method
  * property to allow finer control over burst-on-connect behaviour. By selecting
  * the 'burst' method a minimum burst size can be chosen, 'burst-keyframe'
  * additionally requires that the burst begin with a keyframe, and 
@@ -96,8 +96,6 @@
  * buffers to the clients. This behaviour can be disabled by setting the sync 
  * property to FALSE. Multisocketsink will by default not do QoS and will never
  * drop late buffers.
- *
- * Last reviewed on 2006-09-12 (0.10.10)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -105,6 +103,7 @@
 #endif
 
 #include <gst/gst-i18n-plugin.h>
+#include <gst/net/gstnetcontrolmessagemeta.h>
 
 #include <string.h>
 
@@ -340,8 +339,6 @@ gst_multi_socket_sink_class_init (GstMultiSocketSinkClass * klass)
    * In this callback, @gstmultisocketsink has removed all the information
    * associated with @socket and it is therefore not possible to call get-stats
    * with @socket. It is however safe to close() and reuse @fd in the callback.
-   *
-   * Since: 0.10.7
    */
   gst_multi_socket_sink_signals[SIGNAL_CLIENT_SOCKET_REMOVED] =
       g_signal_new ("client-socket-removed", G_TYPE_FROM_CLASS (klass),
@@ -578,7 +575,7 @@ gst_multi_socket_sink_handle_client_read (GstMultiSocketSink * sink,
     GST_DEBUG_OBJECT (sink, "%s client wants us to read", mhclient->debug);
 
     navail = g_socket_get_available_bytes (mhclient->handle.socket);
-    if (navail < 0)
+    if (navail <= 0)
       break;
 
     nread =
@@ -604,6 +601,120 @@ gst_multi_socket_sink_handle_client_read (GstMultiSocketSink * sink,
   return ret;
 }
 
+/**
+ * map_memory_output_vector_n:
+ * @buf: The #GstBuffer that should be mapped
+ * @offset: Offset into the buffer that should be mapped
+ * @vectors: (out,array length=num_vectors): an array of #GOutputVector structs to write into
+ * @mapinfo: (out,array length=num_vectors): an array of #GstMapInfo structs to write into
+ * @num_vectors: the number of elements in @vectors to prevent buffer overruns
+ *
+ * Maps a buffer into memory, populating a #GOutputVector to use scatter-gather
+ * I/O to send the data over a socket.  The whole buffer won't be mapped into
+ * memory if it consists of more than @num_vectors #GstMemory s.
+ *
+ * Use #unmap_n_memorys after you are
+ * finished with the mappings.
+ *
+ * Returns: The number of GstMemorys mapped
+ */
+static int
+map_n_memory_output_vector (GstBuffer * buf, size_t offset,
+    GOutputVector * vectors, GstMapInfo * mapinfo, int num_vectors)
+{
+  guint mem_idx, mem_len;
+  gsize mem_skip;
+  size_t maxsize;
+  int i;
+
+  g_return_val_if_fail (num_vectors > 0, 0);
+  memset (vectors, 0, sizeof (GOutputVector) * num_vectors);
+
+  maxsize = gst_buffer_get_size (buf) - offset;
+  if (!gst_buffer_find_memory (buf, offset, maxsize, &mem_idx, &mem_len,
+          &mem_skip))
+    g_error ("Unable to map memory at offset %" G_GSIZE_FORMAT ", buffer "
+        "length is %" G_GSIZE_FORMAT, offset, gst_buffer_get_size (buf));
+
+  for (i = 0; i < mem_len && i < num_vectors; i++) {
+    GstMapInfo map = { 0 };
+    GstMemory *mem = gst_buffer_peek_memory (buf, mem_idx + i);
+    if (!gst_memory_map (mem, &map, GST_MAP_READ))
+      g_error ("Unable to map memory %p.  This should never happen.", mem);
+
+    if (i == 0) {
+      vectors[i].buffer = map.data + mem_skip;
+      vectors[i].size = map.size - mem_skip;
+    } else {
+      vectors[i].buffer = map.data;
+      vectors[i].size = map.size;
+    }
+    mapinfo[i] = map;
+  }
+  return i;
+}
+
+/**
+ * map_n_memory_output_vector:
+ * @buf: The #GstBuffer that should be mapped
+ * @offset: Offset into the buffer that should be mapped
+ * @vectors: (out,array length=num_vectors): an array of #GOutputVector structs to write into
+ * @num_vectors: the number of elements in @vectors to prevent buffer overruns
+ *
+ * Returns: The number of GstMemorys mapped
+ */
+static void
+unmap_n_memorys (GstMapInfo * mapinfo, int num_mappings)
+{
+  int i;
+  g_return_if_fail (num_mappings > 0);
+
+  for (i = 0; i < num_mappings; i++)
+    gst_memory_unmap (mapinfo[i].memory, &mapinfo[i]);
+}
+
+static gsize
+gst_buffer_get_cmsg_list (GstBuffer * buf, GSocketControlMessage ** msgs,
+    gsize msg_space)
+{
+  gpointer iter_state = NULL;
+  GstMeta *meta;
+  gsize msg_count = 0;
+
+  while ((meta = gst_buffer_iterate_meta (buf, &iter_state)) != NULL
+      && msg_count < msg_space) {
+    if (meta->info->api == GST_NET_CONTROL_MESSAGE_META_API_TYPE)
+      msgs[msg_count++] = ((GstNetControlMessageMeta *) meta)->message;
+  }
+
+  return msg_count;
+}
+
+#define CMSG_MAX 255
+
+static gssize
+gst_multi_socket_sink_write (GstMultiSocketSink * sink,
+    GSocket * sock, GstBuffer * buffer, gsize bufoffset,
+    GCancellable * cancellable, GError ** err)
+{
+  GstMapInfo maps[8];
+  GOutputVector vec[8];
+  guint mems_mapped;
+  gssize wrote;
+  GSocketControlMessage *cmsgs[CMSG_MAX];
+  gsize msg_count;
+
+  mems_mapped = map_n_memory_output_vector (buffer, bufoffset, vec, maps, 8);
+
+  msg_count = gst_buffer_get_cmsg_list (buffer, cmsgs, CMSG_MAX);
+
+  wrote =
+      g_socket_send_message (sock, NULL, vec, mems_mapped, cmsgs, msg_count, 0,
+      cancellable, err);
+  unmap_n_memorys (maps, mems_mapped);
+  return wrote;
+}
+
 /* Handle a write on a client,
  * which indicates a read request from a client.
  *
@@ -648,8 +759,6 @@ gst_multi_socket_sink_handle_client_write (GstMultiSocketSink * sink,
 
   more = TRUE;
   do {
-    gint maxsize;
-
     if (!mhclient->sending) {
       /* client is not working on a buffer */
       if (mhclient->bufpos == -1) {
@@ -729,39 +838,32 @@ gst_multi_socket_sink_handle_client_write (GstMultiSocketSink * sink,
     if (mhclient->sending) {
       gssize wrote;
       GstBuffer *head;
-      GstMapInfo map;
 
       /* pick first buffer from list */
       head = GST_BUFFER (mhclient->sending->data);
 
-      gst_buffer_map (head, &map, GST_MAP_READ);
-      maxsize = map.size - mhclient->bufoffset;
-
-      /* FIXME: specific */
-      /* try to write the complete buffer */
-
-      wrote =
-          g_socket_send (mhclient->handle.socket,
-          (gchar *) map.data + mhclient->bufoffset, maxsize, sink->cancellable,
-          &err);
-      gst_buffer_unmap (head, &map);
+      wrote = gst_multi_socket_sink_write (sink, mhclient->handle.socket, head,
+          mhclient->bufoffset, sink->cancellable, &err);
 
       if (wrote < 0) {
         /* hmm error.. */
         if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_CLOSED)) {
           goto connection_reset;
+        } else if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
+          /* write would block, try again later */
+          GST_LOG_OBJECT (sink, "write would block %p",
+              mhclient->handle.socket);
+          more = FALSE;
         } else {
           goto write_error;
         }
       } else {
-        if (wrote < maxsize) {
-          /* partial write means that the client cannot read more and we should
-           * stop sending more */
+        if (wrote < (gst_buffer_get_size (head) - mhclient->bufoffset)) {
+          /* partial write, try again now */
           GST_LOG_OBJECT (sink,
               "partial write on %p of %" G_GSSIZE_FORMAT " bytes",
               mhclient->handle.socket, wrote);
           mhclient->bufoffset += wrote;
-          more = FALSE;
         } else {
           /* complete buffer was written, we can proceed to the next one */
           mhclient->sending = g_slist_remove (mhclient->sending, head);
@@ -1070,7 +1172,8 @@ gst_multi_socket_sink_unlock_stop (GstBaseSink * bsink)
   sink = GST_MULTI_SOCKET_SINK (bsink);
 
   GST_DEBUG_OBJECT (sink, "unset flushing");
-  g_cancellable_reset (sink->cancellable);
+  g_object_unref (sink->cancellable);
+  sink->cancellable = g_cancellable_new ();
 
   return TRUE;
 }
