@@ -23,7 +23,6 @@
 
 /**
  * SECTION:element-socketsrc
- * @title: socketsrc
  *
  * Receive data from a socket.
  *
@@ -51,8 +50,8 @@
 
 #include <gst/gst-i18n-plugin.h>
 #include <gst/net/gstnetcontrolmessagemeta.h>
-#include "gsttcpelements.h"
 #include "gstsocketsrc.h"
+#include "gsttcp.h"
 
 GST_DEBUG_CATEGORY_STATIC (socketsrc_debug);
 #define GST_CAT_DEFAULT socketsrc_debug
@@ -66,14 +65,10 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS_ANY);
 
 
-#define DEFAULT_SEND_MESSAGES FALSE
-
 enum
 {
   PROP_0,
   PROP_SOCKET,
-  PROP_CAPS,
-  PROP_SEND_MESSAGES
 };
 
 enum
@@ -86,13 +81,10 @@ static guint gst_socket_src_signals[LAST_SIGNAL] = { 0 };
 
 #define gst_socket_src_parent_class parent_class
 G_DEFINE_TYPE (GstSocketSrc, gst_socket_src, GST_TYPE_PUSH_SRC);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (socketsrc, "socketsrc",
-    GST_RANK_NONE, GST_TYPE_SOCKET_SRC, tcp_element_init (plugin));
+
 
 static void gst_socket_src_finalize (GObject * gobject);
 
-static GstCaps *gst_socketsrc_getcaps (GstBaseSrc * src, GstCaps * filter);
-static gboolean gst_socketsrc_event (GstBaseSrc * src, GstEvent * event);
 static GstFlowReturn gst_socket_src_fill (GstPushSrc * psrc,
     GstBuffer * outbuf);
 static gboolean gst_socket_src_unlock (GstBaseSrc * bsrc);
@@ -127,35 +119,13 @@ gst_socket_src_class_init (GstSocketSrcClass * klass)
           "The socket to receive packets from", G_TYPE_SOCKET,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, PROP_CAPS,
-      g_param_spec_boxed ("caps", "Caps",
-          "The caps of the source pad", GST_TYPE_CAPS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GstSocketSrc:send-messages:
-   *
-   * Control if the source will handle GstNetworkMessage events.
-   * The event is a CUSTOM event named 'GstNetworkMessage' and contains:
-   *
-   *   "buffer", GST_TYPE_BUFFER    : the buffer with data to send
-   *
-   * The buffer in the event will be sent on the socket. This allows
-   * for simple bidirectional communication.
-   *
-   * Since: 1.8.0
-   **/
-  g_object_class_install_property (gobject_class, PROP_SEND_MESSAGES,
-      g_param_spec_boolean ("send-messages", "Send Messages",
-          "If GstNetworkMessage events should be handled",
-          DEFAULT_SEND_MESSAGES, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
   gst_socket_src_signals[CONNECTION_CLOSED_BY_PEER] =
       g_signal_new ("connection-closed-by-peer", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (GstSocketSrcClass,
           connection_closed_by_peer), NULL, NULL, NULL, G_TYPE_NONE, 0);
 
-  gst_element_class_add_static_pad_template (gstelement_class, &srctemplate);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&srctemplate));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "socket source", "Source/Network",
@@ -163,8 +133,6 @@ gst_socket_src_class_init (GstSocketSrcClass * klass)
       "Thomas Vander Stichele <thomas at apestaart dot org>, "
       "William Manley <will@williammanley.net>");
 
-  gstbasesrc_class->event = gst_socketsrc_event;
-  gstbasesrc_class->get_caps = gst_socketsrc_getcaps;
   gstbasesrc_class->unlock = gst_socket_src_unlock;
   gstbasesrc_class->unlock_stop = gst_socket_src_unlock_stop;
 
@@ -178,7 +146,6 @@ gst_socket_src_init (GstSocketSrc * this)
 {
   this->socket = NULL;
   this->cancellable = g_cancellable_new ();
-  this->send_messages = DEFAULT_SEND_MESSAGES;
 }
 
 static void
@@ -186,93 +153,12 @@ gst_socket_src_finalize (GObject * gobject)
 {
   GstSocketSrc *this = GST_SOCKET_SRC (gobject);
 
-  if (this->caps)
-    gst_caps_unref (this->caps);
   g_clear_object (&this->cancellable);
   g_clear_object (&this->socket);
 
   G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
-static gboolean
-gst_socketsrc_event (GstBaseSrc * bsrc, GstEvent * event)
-{
-  GstSocketSrc *src;
-  gboolean res = FALSE;
-
-  src = GST_SOCKET_SRC (bsrc);
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_CUSTOM_UPSTREAM:
-      if (src->send_messages && gst_event_has_name (event, "GstNetworkMessage")) {
-        const GstStructure *str = gst_event_get_structure (event);
-        GSocket *socket;
-
-        GST_OBJECT_LOCK (src);
-        if ((socket = src->socket))
-          g_object_ref (socket);
-        GST_OBJECT_UNLOCK (src);
-
-        if (socket) {
-          GstBuffer *buf;
-          GstMapInfo map;
-          GError *err = NULL;
-          gssize ret;
-
-          gst_structure_get (str, "buffer", GST_TYPE_BUFFER, &buf, NULL);
-
-          if (buf) {
-            gst_buffer_map (buf, &map, GST_MAP_READ);
-            GST_LOG ("sending buffer of size %" G_GSIZE_FORMAT, map.size);
-            ret = g_socket_send_with_blocking (socket, (gchar *) map.data,
-                map.size, FALSE, src->cancellable, &err);
-            gst_buffer_unmap (buf, &map);
-
-            if (ret == -1) {
-              GST_WARNING ("could not send message: %s", err->message);
-              g_clear_error (&err);
-              res = FALSE;
-            } else
-              res = TRUE;
-            gst_buffer_unref (buf);
-          }
-          g_object_unref (socket);
-        }
-      }
-      break;
-    default:
-      res = GST_BASE_SRC_CLASS (parent_class)->event (bsrc, event);
-      break;
-  }
-  return res;
-}
-
-static GstCaps *
-gst_socketsrc_getcaps (GstBaseSrc * src, GstCaps * filter)
-{
-  GstSocketSrc *socketsrc;
-  GstCaps *caps, *result;
-
-  socketsrc = GST_SOCKET_SRC (src);
-
-  GST_OBJECT_LOCK (src);
-  if ((caps = socketsrc->caps))
-    gst_caps_ref (caps);
-  GST_OBJECT_UNLOCK (src);
-
-  if (caps) {
-    if (filter) {
-      result = gst_caps_intersect_full (filter, caps, GST_CAPS_INTERSECT_FIRST);
-      gst_caps_unref (caps);
-    } else {
-      result = caps;
-    }
-  } else {
-    result = (filter) ? gst_caps_ref (filter) : gst_caps_new_any ();
-  }
-  return result;
-}
-
 static GstFlowReturn
 gst_socket_src_fill (GstPushSrc * psrc, GstBuffer * outbuf)
 {
@@ -399,32 +285,6 @@ gst_socket_src_set_property (GObject * object, guint prop_id,
       g_clear_object (&socket);
       break;
     }
-    case PROP_CAPS:
-    {
-      const GstCaps *new_caps_val = gst_value_get_caps (value);
-      GstCaps *new_caps;
-      GstCaps *old_caps;
-
-      if (new_caps_val == NULL) {
-        new_caps = gst_caps_new_any ();
-      } else {
-        new_caps = gst_caps_copy (new_caps_val);
-      }
-
-      GST_OBJECT_LOCK (socketsrc);
-      old_caps = socketsrc->caps;
-      socketsrc->caps = new_caps;
-      GST_OBJECT_UNLOCK (socketsrc);
-
-      if (old_caps)
-        gst_caps_unref (old_caps);
-
-      gst_pad_mark_reconfigure (GST_BASE_SRC_PAD (socketsrc));
-      break;
-    }
-    case PROP_SEND_MESSAGES:
-      socketsrc->send_messages = g_value_get_boolean (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -441,14 +301,6 @@ gst_socket_src_get_property (GObject * object, guint prop_id,
     case PROP_SOCKET:
       g_value_set_object (value, socketsrc->socket);
       break;
-    case PROP_CAPS:
-      GST_OBJECT_LOCK (socketsrc);
-      gst_value_set_caps (value, socketsrc->caps);
-      GST_OBJECT_UNLOCK (socketsrc);
-      break;
-    case PROP_SEND_MESSAGES:
-      g_value_set_boolean (value, socketsrc->send_messages);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
