@@ -192,8 +192,6 @@ GST_START_TEST (test_musicbrainz_tag_registration)
   gst_vorbis_tag_add (list, "MUSICBRAINZ_ARTISTID", "234567");
   gst_vorbis_tag_add (list, "MUSICBRAINZ_ALBUMID", "345678");
   gst_vorbis_tag_add (list, "MUSICBRAINZ_ALBUMARTISTID", "4567890");
-  gst_vorbis_tag_add (list, "MUSICBRAINZ_RELEASETRACKID", "4567891");
-  gst_vorbis_tag_add (list, "MUSICBRAINZ_RELEASEGROUPID", "4567892");
   gst_vorbis_tag_add (list, "MUSICBRAINZ_TRMID", "5678901");
   /* MUSICBRAINZ_SORTNAME = GST_TAG_ARTIST_SORTNAME now */
   gst_vorbis_tag_add (list, "MUSICBRAINZ_SORTNAME", "Five, 678901");
@@ -203,10 +201,6 @@ GST_START_TEST (test_musicbrainz_tag_registration)
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_ALBUMID, "345678");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_ALBUMARTISTID,
       "4567890");
-  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_RELEASETRACKID,
-      "4567891");
-  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_RELEASEGROUPID,
-      "4567892");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_TRMID, "5678901");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ARTIST_SORTNAME, "Five, 678901");
 
@@ -253,8 +247,6 @@ GST_START_TEST (test_vorbis_tags)
   gst_vorbis_tag_add (list, "CONTACT", "Coo");
   gst_vorbis_tag_add (list, "COMMENT", "Stroodle is good");
   gst_vorbis_tag_add (list, "COMMENT", "Peroxysulfid stroodles the brain");
-  gst_vorbis_tag_add (list, "ACOUSTID_ID", "5678913");
-  gst_vorbis_tag_add (list, "ACOUSTID_FINGERPRINT", "5678912");
 
   gst_vorbis_tag_add (list, "TRACKNUMBER", "5");
   gst_vorbis_tag_add (list, "TRACKTOTAL", "77");
@@ -277,8 +269,6 @@ GST_START_TEST (test_vorbis_tags)
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_CONTACT, "Coo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_COMMENT,
       "Peroxysulfid stroodles the brain");
-  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ACOUSTID_ID, "5678913");
-  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ACOUSTID_FINGERPRINT, "5678912");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_COMMENT, "Stroodle is good");
   ASSERT_TAG_LIST_HAS_UINT (list, GST_TAG_TRACK_NUMBER, 5);
   ASSERT_TAG_LIST_HAS_UINT (list, GST_TAG_TRACK_COUNT, 77);
@@ -696,168 +686,6 @@ GST_START_TEST (test_id3v1_utf8_tag)
 
 GST_END_TEST;
 
-GST_START_TEST (test_id3v2_priv_tag)
-{
-  const guint8 id3v2[] = {
-    0x49, 0x44, 0x33, 0x04, 0x00, 0x00, 0x00, 0x00,
-    0x00, 0x3f, 0x50, 0x52, 0x49, 0x56, 0x00, 0x00,
-    0x00, 0x35, 0x00, 0x00, 0x63, 0x6f, 0x6d, 0x2e,
-    0x61, 0x70, 0x70, 0x6c, 0x65, 0x2e, 0x73, 0x74,
-    0x72, 0x65, 0x61, 0x6d, 0x69, 0x6e, 0x67, 0x2e,
-    0x74, 0x72, 0x61, 0x6e, 0x73, 0x70, 0x6f, 0x72,
-    0x74, 0x53, 0x74, 0x72, 0x65, 0x61, 0x6d, 0x54,
-    0x69, 0x6d, 0x65, 0x73, 0x74, 0x61, 0x6d, 0x70,
-    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0d, 0xbb,
-    0xa0
-  };
-  const GstStructure *s;
-  GstTagList *tags;
-  GstSample *sample = NULL;
-  GstBuffer *buf;
-  GstMapInfo map;
-  gchar *owner = NULL;
-
-  buf = gst_buffer_new_allocate (NULL, sizeof (id3v2), NULL);
-  gst_buffer_fill (buf, 0, id3v2, sizeof (id3v2));
-
-  tags = gst_tag_list_from_id3v2_tag (buf);
-  gst_buffer_unref (buf);
-
-  fail_if (tags == NULL, "Failed to parse ID3 tag");
-
-  GST_LOG ("tags: %" GST_PTR_FORMAT, tags);
-
-  if (!gst_tag_list_get_sample (tags, GST_TAG_PRIVATE_DATA, &sample))
-    g_error ("Failed to get PRIVATE_DATA tag");
-
-  s = gst_sample_get_info (sample);
-  buf = gst_sample_get_buffer (sample);
-
-  if (!gst_structure_has_name (s, "ID3PrivateFrame"))
-    g_error ("wrong info name");
-
-  gst_structure_get (s, "owner", G_TYPE_STRING, &owner, NULL);
-  fail_unless (owner != NULL);
-
-  fail_unless_equals_string (owner,
-      "com.apple.streaming.transportStreamTimestamp");
-
-  fail_unless_equals_int (gst_buffer_get_size (buf), 8);
-
-  gst_buffer_map (buf, &map, GST_MAP_READ);
-  GST_MEMDUMP ("private data", map.data, map.size);
-  fail_unless_equals_uint64 (GST_READ_UINT64_BE (map.data), 0x0dbba0);
-  gst_buffer_unmap (buf, &map);
-  g_free (owner);
-
-  gst_sample_unref (sample);
-  gst_tag_list_unref (tags);
-}
-
-GST_END_TEST;
-
-static GstTagList *
-parse_id3v2_tag_from_data (const guint8 * id3v2, gsize id3v2_size)
-{
-  GstTagList *tags;
-  GstBuffer *buf;
-
-  GST_MEMDUMP ("id3v2 tag", id3v2, id3v2_size);
-
-  buf = gst_buffer_new_allocate (NULL, id3v2_size, NULL);
-  gst_buffer_fill (buf, 0, id3v2, id3v2_size);
-  tags = gst_tag_list_from_id3v2_tag (buf);
-  gst_buffer_unref (buf);
-
-  return tags;
-}
-
-GST_START_TEST (test_id3v2_extended_header)
-{
-  const guint8 id3v24_exthdr[1024] = {
-    0x49, 0x44, 0x33, 0x04, 0x00, 0x40, 0x00, 0x00, 0x07, 0x76,
-    0x00, 0x00, 0x00, 0x0c, 0x01, 0x20, 0x05, 0x0b, 0x7f, 0x06,
-    0x43, 0x22, 0x54, 0x53, 0x53, 0x45, 0x00, 0x00, 0x00, 0x0e,
-    0x00, 0x00, 0x03, 0x4c, 0x61, 0x76, 0x66, 0x35, 0x37, 0x2e,
-    0x37, 0x31, 0x2e, 0x31, 0x30, 0x30, 0x54, 0x49, 0x54, 0x32,
-    0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x03, 0x53, 0x69, 0x6c,
-    0x65, 0x6e, 0x63, 0x65, 0x54, 0x50, 0x45, 0x31, 0x00, 0x00,
-    0x00, 0x07, 0x00, 0x00, 0x03, 0x4e, 0x6f, 0x20, 0x6f, 0x6e,
-    0x65, 0x54, 0x50, 0x45, 0x32, 0x00, 0x00, 0x00, 0x05, 0x00,
-    0x00, 0x03, 0x4e, 0x6f, 0x6e, 0x65, 0x54, 0x41, 0x4c, 0x42,
-    0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x03, 0x4e, 0x65, 0x69,
-    0x74, 0x68, 0x65, 0x72, 0x54, 0x44, 0x52, 0x43, 0x00, 0x00,
-    0x00, 0x05, 0x00, 0x00, 0x03, 0x32, 0x30, 0x31, 0x38, 0x54,
-    0x52, 0x43, 0x4b, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x03,
-    0x30, 0x31, 0x2f, 0x30, 0x31, 0x54, 0x43, 0x4f, 0x4e, 0x00,
-    0x00, 0x00, 0x06, 0x00, 0x00, 0x03, 0x28, 0x31, 0x34, 0x38,
-    0x29,
-  };
-  const guint8 id3v2_exthdr[] = {
-    0x49, 0x44, 0x33, 0x03, 0x00, 0x40, 0x00, 0x00, 0x00, 0x1b,
-    0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-    0x54, 0x50, 0x45, 0x31, 0x00, 0x00, 0x00, 0x07, 0x00, 0x00,
-    0x00, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x65
-  };
-  const guint8 id3v2_no_exthdr[] = {
-    0x49, 0x44, 0x33, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11,
-    0x54, 0x50, 0x45, 0x31, 0x00, 0x00, 0x00, 0x07, 0x00, 0x00,
-    0x00, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x65
-  };
-  GstTagList *tags;
-
-  tags = parse_id3v2_tag_from_data (id3v2_exthdr, sizeof (id3v2_exthdr));
-  fail_if (tags == NULL, "Failed to parse ID3 v2.3 tag with extension header");
-  GST_LOG ("tags: %" GST_PTR_FORMAT, tags);
-  fail_unless_equals_int (gst_tag_list_n_tags (tags), 1);
-  gst_tag_list_unref (tags);
-
-  tags = parse_id3v2_tag_from_data (id3v24_exthdr, sizeof (id3v24_exthdr));
-  fail_if (tags == NULL, "Failed to parse ID3 v2.4 tag with extension header");
-  GST_LOG ("tags: %" GST_PTR_FORMAT, tags);
-  fail_unless_equals_int (gst_tag_list_n_tags (tags), 9);
-  gst_tag_list_unref (tags);
-
-  tags = parse_id3v2_tag_from_data (id3v2_no_exthdr, sizeof (id3v2_no_exthdr));
-  fail_if (tags == NULL, "Failed to parse ID3 tag without extension header");
-  GST_LOG ("tags: %" GST_PTR_FORMAT, tags);
-  fail_unless_equals_int (gst_tag_list_n_tags (tags), 1);
-  gst_tag_list_unref (tags);
-}
-
-GST_END_TEST;
-
-GST_START_TEST (test_id3v2_string_list_utf16)
-{
-  const guint8 id3v2[] = {
-    0x49, 0x44, 0x33, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40,
-    0x43, 0x4f, 0x4d, 0x4d, 0x00, 0x00, 0x00, 0x36, 0x00, 0x00,
-    0x01, 0x65, 0x6e, 0x67, 0xff, 0xfe, 0x00, 0x00, 0xff, 0xfe,
-    0x4e, 0x00, 0x61, 0x00, 0x69, 0x00, 0x6d, 0x00, 0x20, 0x00,
-    0x4d, 0x00, 0x50, 0x00, 0x33, 0x00, 0x20, 0x00, 0x4d, 0x00,
-    0x75, 0x00, 0x73, 0x00, 0x69, 0x00, 0x63, 0x00, 0x20, 0x00,
-    0x4c, 0x00, 0x69, 0x00, 0x62, 0x00, 0x72, 0x00, 0x61, 0x00,
-    0x72, 0x00, 0x79, 0x00
-  };
-  GstTagList *tags;
-  gchar *comment = NULL;
-
-  tags = parse_id3v2_tag_from_data (id3v2, sizeof (id3v2));
-  fail_if (tags == NULL,
-      "Failed to parse ID3 tag with UTF-16 strings and BOMs");
-
-  GST_LOG ("tags: %" GST_PTR_FORMAT, tags);
-
-  gst_tag_list_get_string (tags, GST_TAG_COMMENT, &comment);
-  fail_unless (comment != NULL, "Expected comment tag");
-  GST_MEMDUMP ("comment string UTF-8", (guint8 *) comment, strlen (comment));
-  fail_unless_equals_string (comment, "Naim MP3 Music Library");
-  g_free (comment);
-  gst_tag_list_unref (tags);
-}
-
-GST_END_TEST;
-
 GST_START_TEST (test_language_utils)
 {
   gchar **lang_codes, **c;
@@ -891,7 +719,7 @@ GST_START_TEST (test_language_utils)
     ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_1 (c2t), *c);
     ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_1 (c2b), *c);
 
-    GST_DEBUG ("[%s] %s %s %s : %s", *c, c1, c2t, c2b, lang_name);
+    GST_DEBUG ("[%s] %s %s %s : %s\n", *c, c1, c2t, c2b, lang_name);
 
   }
   g_strfreev (lang_codes);
@@ -1131,7 +959,7 @@ GST_START_TEST (test_xmp_formatting)
 
   /* test data */
   list = gst_tag_list_new (GST_TAG_TITLE, "test title",
-      GST_TAG_DESCRIPTION, "test description",
+      GST_TAG_DESCRIPTION, "test decription",
       GST_TAG_KEYWORDS, "keyword1", GST_TAG_KEYWORDS, "keyword2", NULL);
 
   buf = gst_tag_list_to_xmp_buffer (list, FALSE, NULL);
@@ -1144,7 +972,7 @@ GST_START_TEST (test_xmp_formatting)
   /* check the content */
   fail_unless (g_strrstr_len (text, len, "<?xpacket begin") == text);
   fail_unless (g_strrstr_len (text, len, ">test title<") != NULL);
-  fail_unless (g_strrstr_len (text, len, ">test description<") != NULL);
+  fail_unless (g_strrstr_len (text, len, ">test decription<") != NULL);
   fail_unless (g_strrstr_len (text, len, ">keyword1<") != NULL);
   fail_unless (g_strrstr_len (text, len, ">keyword2<") != NULL);
   fail_unless (g_strrstr_len (text, len, "<?xpacket end") != NULL);
@@ -1183,15 +1011,15 @@ GST_START_TEST (test_xmp_parsing)
     gint result_test;
   } test_data[] = {
     {
-        "", -1, -1}, {
-        "<rdf:Description rdf:about=\"\" />", 0, -1}, {
-        "<rdf:Description rdf:about=\"\"></rdf:Description>", 0, -1}, {
-        "<rdf:Description    rdf:about=\"\"    ></rdf:Description>", 0, -1}, {
-          "<rdf:Description rdf:about=\"\"><dc:description>test</dc:description></rdf:Description>",
-        1, 0}, {
-          "<rdf:Description rdf:about=\"\" dc:description=\"test\"></rdf:Description>",
-        1, 0}, {
-        NULL, -1, -1}
+    "", -1, -1}, {
+    "<rdf:Description rdf:about=\"\" />", 0, -1}, {
+    "<rdf:Description rdf:about=\"\"></rdf:Description>", 0, -1}, {
+    "<rdf:Description    rdf:about=\"\"    ></rdf:Description>", 0, -1}, {
+    "<rdf:Description rdf:about=\"\"><dc:description>test</dc:description></rdf:Description>",
+          1, 0}, {
+    "<rdf:Description rdf:about=\"\" dc:description=\"test\"></rdf:Description>",
+          1, 0}, {
+    NULL, -1, -1}
   };
 
   /* test data */
@@ -1789,11 +1617,10 @@ GST_START_TEST (test_exif_tags_serialization_deserialization)
   g_value_unset (&value);
 
   g_value_init (&value, G_TYPE_DOUBLE);
-  g_value_set_double (&value, 40.3456784);
+  g_value_set_double (&value, 30.5);
   do_simple_exif_tag_serialization_deserialization
       (GST_TAG_GEO_LOCATION_LATITUDE, &value);
-  g_value_set_double (&value, -12.1250865);
-
+  g_value_set_double (&value, -12.125);
   do_simple_exif_tag_serialization_deserialization
       (GST_TAG_GEO_LOCATION_LATITUDE, &value);
   g_value_set_double (&value, 0);
@@ -1889,10 +1716,6 @@ GST_START_TEST (test_exif_tags_serialization_deserialization)
   g_value_set_double (&value, -2.5);
   do_simple_exif_tag_serialization_deserialization
       (GST_TAG_CAPTURING_EXPOSURE_COMPENSATION, &value);
-
-  g_value_set_double (&value, 50.0);
-  do_simple_exif_tag_serialization_deserialization
-      (GST_TAG_CAPTURING_FOCAL_LENGTH_35_MM, &value);
   g_value_unset (&value);
 
   g_value_init (&value, G_TYPE_INT);
@@ -1976,9 +1799,6 @@ tag_suite (void)
   tcase_add_test (tc_chain, test_vorbis_tags);
   tcase_add_test (tc_chain, test_id3_tags);
   tcase_add_test (tc_chain, test_id3v1_utf8_tag);
-  tcase_add_test (tc_chain, test_id3v2_priv_tag);
-  tcase_add_test (tc_chain, test_id3v2_extended_header);
-  tcase_add_test (tc_chain, test_id3v2_string_list_utf16);
   tcase_add_test (tc_chain, test_language_utils);
   tcase_add_test (tc_chain, test_license_utils);
   tcase_add_test (tc_chain, test_xmp_formatting);
